from __future__ import unicode_literals
import boto
import boto3
import boto.ec2.autoscale
from boto.ec2.autoscale.launchconfig import LaunchConfiguration
from boto.ec2.autoscale.group import AutoScalingGroup
from boto.ec2.autoscale import Tag
import boto.ec2.elb
import sure  # noqa

from moto import mock_autoscaling, mock_ec2_deprecated, mock_elb_deprecated, mock_elb, mock_autoscaling_deprecated, mock_ec2
from tests.helpers import requires_boto_gte

from utils import setup_networking, setup_networking_deprecated


@mock_autoscaling_deprecated
@mock_elb_deprecated
def test_create_autoscaling_group():
    mocked_networking = setup_networking_deprecated()
    elb_conn = boto.ec2.elb.connect_to_region('us-east-1')
    elb_conn.create_load_balancer(
        'test_lb', zones=[], listeners=[(80, 8080, 'http')])

    conn = boto.ec2.autoscale.connect_to_region('us-east-1')
    config = LaunchConfiguration(
        name='tester',
        image_id='ami-abcd1234',
        instance_type='t2.medium',
    )
    conn.create_launch_configuration(config)

    group = AutoScalingGroup(
        name='tester_group',
        availability_zones=['us-east-1a', 'us-east-1b'],
        default_cooldown=60,
        desired_capacity=2,
        health_check_period=100,
        health_check_type="EC2",
        max_size=2,
        min_size=2,
        launch_config=config,
        load_balancers=["test_lb"],
        placement_group="test_placement",
        vpc_zone_identifier="{subnet1},{subnet2}".format(
            subnet1=mocked_networking['subnet1'],
            subnet2=mocked_networking['subnet2'],
        ),
        termination_policies=["OldestInstance", "NewestInstance"],
        tags=[Tag(
            resource_id='tester_group',
            key='test_key',
            value='test_value',
            propagate_at_launch=True
        )
        ],
    )
    conn.create_auto_scaling_group(group)

    group = conn.get_all_groups()[0]
    group.name.should.equal('tester_group')
    set(group.availability_zones).should.equal(
        set(['us-east-1a', 'us-east-1b']))
    group.desired_capacity.should.equal(2)
    group.max_size.should.equal(2)
    group.min_size.should.equal(2)
    group.instances.should.have.length_of(2)
    group.vpc_zone_identifier.should.equal("{subnet1},{subnet2}".format(
        subnet1=mocked_networking['subnet1'],
        subnet2=mocked_networking['subnet2'],
    ))
    group.launch_config_name.should.equal('tester')
    group.default_cooldown.should.equal(60)
    group.health_check_period.should.equal(100)
    group.health_check_type.should.equal("EC2")
    list(group.load_balancers).should.equal(["test_lb"])
    group.placement_group.should.equal("test_placement")
    list(group.termination_policies).should.equal(
        ["OldestInstance", "NewestInstance"])
    len(list(group.tags)).should.equal(1)
    tag = list(group.tags)[0]
    tag.resource_id.should.equal('tester_group')
    tag.key.should.equal('test_key')
    tag.value.should.equal('test_value')
    tag.propagate_at_launch.should.equal(True)


@mock_autoscaling_deprecated
def test_create_autoscaling_groups_defaults():
    """ Test with the minimum inputs and check that all of the proper defaults
    are assigned for the other attributes """

    mocked_networking = setup_networking_deprecated()
    conn = boto.connect_autoscale()
    config = LaunchConfiguration(
        name='tester',
        image_id='ami-abcd1234',
        instance_type='t2.medium',
    )
    conn.create_launch_configuration(config)

    group = AutoScalingGroup(
        name='tester_group',
        max_size=2,
        min_size=2,
        launch_config=config,
        vpc_zone_identifier=mocked_networking['subnet1'],
    )
    conn.create_auto_scaling_group(group)

    group = conn.get_all_groups()[0]
    group.name.should.equal('tester_group')
    group.max_size.should.equal(2)
    group.min_size.should.equal(2)
    group.launch_config_name.should.equal('tester')

    # Defaults
    list(group.availability_zones).should.equal(['us-east-1a'])  # subnet1
    group.desired_capacity.should.equal(2)
    group.vpc_zone_identifier.should.equal(mocked_networking['subnet1'])
    group.default_cooldown.should.equal(300)
    group.health_check_period.should.equal(300)
    group.health_check_type.should.equal("EC2")
    list(group.load_balancers).should.equal([])
    group.placement_group.should.equal(None)
    list(group.termination_policies).should.equal([])
    list(group.tags).should.equal([])


@mock_autoscaling
def test_list_many_autoscaling_groups():
    mocked_networking = setup_networking()
    conn = boto3.client('autoscaling', region_name='us-east-1')
    conn.create_launch_configuration(LaunchConfigurationName='TestLC')

    for i in range(51):
        conn.create_auto_scaling_group(AutoScalingGroupName='TestGroup%d' % i,
                                       MinSize=1,
                                       MaxSize=2,
                                       LaunchConfigurationName='TestLC',
                                       VPCZoneIdentifier=mocked_networking['subnet1'])

    response = conn.describe_auto_scaling_groups()
    groups = response["AutoScalingGroups"]
    marker = response["NextToken"]
    groups.should.have.length_of(50)
    marker.should.equal(groups[-1]['AutoScalingGroupName'])

    response2 = conn.describe_auto_scaling_groups(NextToken=marker)

    groups.extend(response2["AutoScalingGroups"])
    groups.should.have.length_of(51)
    assert 'NextToken' not in response2.keys()


@mock_autoscaling
@mock_ec2
def test_list_many_autoscaling_groups():
    mocked_networking = setup_networking()
    conn = boto3.client('autoscaling', region_name='us-east-1')
    conn.create_launch_configuration(LaunchConfigurationName='TestLC')

    conn.create_auto_scaling_group(AutoScalingGroupName='TestGroup1',
                                   MinSize=1,
                                   MaxSize=2,
                                   LaunchConfigurationName='TestLC',
                                   Tags=[{
                                       "ResourceId": 'TestGroup1',
                                       "ResourceType": "auto-scaling-group",
                                       "PropagateAtLaunch": True,
                                       "Key": 'TestTagKey1',
                                       "Value": 'TestTagValue1'
                                   }],
                                   VPCZoneIdentifier=mocked_networking['subnet1'])

    ec2 = boto3.client('ec2', region_name='us-east-1')
    instances = ec2.describe_instances()

    tags = instances['Reservations'][0]['Instances'][0]['Tags']
    tags.should.contain({u'Value': 'TestTagValue1', u'Key': 'TestTagKey1'})
    tags.should.contain({u'Value': 'TestGroup1', u'Key': 'aws:autoscaling:groupName'})


@mock_autoscaling_deprecated
def test_autoscaling_group_describe_filter():
    mocked_networking = setup_networking_deprecated()
    conn = boto.connect_autoscale()
    config = LaunchConfiguration(
        name='tester',
        image_id='ami-abcd1234',
        instance_type='t2.medium',
    )
    conn.create_launch_configuration(config)

    group = AutoScalingGroup(
        name='tester_group',
        max_size=2,
        min_size=2,
        launch_config=config,
        vpc_zone_identifier=mocked_networking['subnet1'],
    )
    conn.create_auto_scaling_group(group)
    group.name = 'tester_group2'
    conn.create_auto_scaling_group(group)
    group.name = 'tester_group3'
    conn.create_auto_scaling_group(group)

    conn.get_all_groups(
        names=['tester_group', 'tester_group2']).should.have.length_of(2)
    conn.get_all_groups().should.have.length_of(3)


@mock_autoscaling_deprecated
def test_autoscaling_update():
    mocked_networking = setup_networking_deprecated()
    conn = boto.connect_autoscale()
    config = LaunchConfiguration(
        name='tester',
        image_id='ami-abcd1234',
        instance_type='t2.medium',
    )
    conn.create_launch_configuration(config)

    group = AutoScalingGroup(
        name='tester_group',
        desired_capacity=2,
        max_size=2,
        min_size=2,
        launch_config=config,
        vpc_zone_identifier=mocked_networking['subnet1'],
    )
    conn.create_auto_scaling_group(group)

    group = conn.get_all_groups()[0]
    group.availability_zones.should.equal(['us-east-1a'])
    group.vpc_zone_identifier.should.equal(mocked_networking['subnet1'])

    group.availability_zones = ['us-east-1b']
    group.vpc_zone_identifier = mocked_networking['subnet2']
    group.update()

    group = conn.get_all_groups()[0]
    group.availability_zones.should.equal(['us-east-1b'])
    group.vpc_zone_identifier.should.equal(mocked_networking['subnet2'])


@mock_autoscaling_deprecated
def test_autoscaling_tags_update():
    mocked_networking = setup_networking_deprecated()
    conn = boto.connect_autoscale()
    config = LaunchConfiguration(
        name='tester',
        image_id='ami-abcd1234',
        instance_type='t2.medium',
    )
    conn.create_launch_configuration(config)

    group = AutoScalingGroup(
        name='tester_group',
        availability_zones=['us-east-1a'],
        desired_capacity=2,
        max_size=2,
        min_size=2,
        launch_config=config,
        tags=[Tag(
            resource_id='tester_group',
            key='test_key',
            value='test_value',
            propagate_at_launch=True
        )],
        vpc_zone_identifier=mocked_networking['subnet1'],
    )
    conn.create_auto_scaling_group(group)

    conn.create_or_update_tags(tags=[Tag(
        resource_id='tester_group',
        key='test_key',
        value='new_test_value',
        propagate_at_launch=True
    ), Tag(
        resource_id='tester_group',
        key='test_key2',
        value='test_value2',
        propagate_at_launch=True
    )])
    group = conn.get_all_groups()[0]
    group.tags.should.have.length_of(2)


@mock_autoscaling_deprecated
def test_autoscaling_group_delete():
    mocked_networking = setup_networking_deprecated()
    conn = boto.connect_autoscale()
    config = LaunchConfiguration(
        name='tester',
        image_id='ami-abcd1234',
        instance_type='t2.medium',
    )
    conn.create_launch_configuration(config)

    group = AutoScalingGroup(
        name='tester_group',
        max_size=2,
        min_size=2,
        launch_config=config,
        vpc_zone_identifier=mocked_networking['subnet1'],
    )
    conn.create_auto_scaling_group(group)

    conn.get_all_groups().should.have.length_of(1)

    conn.delete_auto_scaling_group('tester_group')
    conn.get_all_groups().should.have.length_of(0)


@mock_ec2_deprecated
@mock_autoscaling_deprecated
def test_autoscaling_group_describe_instances():
    mocked_networking = setup_networking_deprecated()
    conn = boto.ec2.autoscale.connect_to_region('us-east-1')
    config = LaunchConfiguration(
        name='tester',
        image_id='ami-abcd1234',
        instance_type='t2.medium',
    )
    conn.create_launch_configuration(config)

    group = AutoScalingGroup(
        name='tester_group',
        max_size=2,
        min_size=2,
        launch_config=config,
        vpc_zone_identifier=mocked_networking['subnet1'],
    )
    conn.create_auto_scaling_group(group)

    instances = list(conn.get_all_autoscaling_instances())
    instances.should.have.length_of(2)
    instances[0].launch_config_name.should.equal('tester')
    instances[0].health_status.should.equal('Healthy')
    autoscale_instance_ids = [instance.instance_id for instance in instances]

    ec2_conn = boto.ec2.connect_to_region('us-east-1')
    reservations = ec2_conn.get_all_instances()
    instances = reservations[0].instances
    instances.should.have.length_of(2)
    instance_ids = [instance.id for instance in instances]
    set(autoscale_instance_ids).should.equal(set(instance_ids))
    instances[0].instance_type.should.equal("t2.medium")


@requires_boto_gte("2.8")
@mock_autoscaling_deprecated
def test_set_desired_capacity_up():
    mocked_networking = setup_networking_deprecated()
    conn = boto.connect_autoscale()
    config = LaunchConfiguration(
        name='tester',
        image_id='ami-abcd1234',
        instance_type='t2.medium',
    )
    conn.create_launch_configuration(config)

    group = AutoScalingGroup(
        name='tester_group',
        availability_zones=['us-east-1a'],
        desired_capacity=2,
        max_size=2,
        min_size=2,
        launch_config=config,
        vpc_zone_identifier=mocked_networking['subnet1'],
    )
    conn.create_auto_scaling_group(group)

    group = conn.get_all_groups()[0]
    group.desired_capacity.should.equal(2)
    instances = list(conn.get_all_autoscaling_instances())
    instances.should.have.length_of(2)

    conn.set_desired_capacity("tester_group", 3)
    group = conn.get_all_groups()[0]
    group.desired_capacity.should.equal(3)

    instances = list(conn.get_all_autoscaling_instances())
    instances.should.have.length_of(3)


@requires_boto_gte("2.8")
@mock_autoscaling_deprecated
def test_set_desired_capacity_down():
    mocked_networking = setup_networking_deprecated()
    conn = boto.connect_autoscale()
    config = LaunchConfiguration(
        name='tester',
        image_id='ami-abcd1234',
        instance_type='t2.medium',
    )
    conn.create_launch_configuration(config)

    group = AutoScalingGroup(
        name='tester_group',
        availability_zones=['us-east-1a'],
        desired_capacity=2,
        max_size=2,
        min_size=2,
        launch_config=config,
        vpc_zone_identifier=mocked_networking['subnet1'],
    )
    conn.create_auto_scaling_group(group)

    group = conn.get_all_groups()[0]
    group.desired_capacity.should.equal(2)
    instances = list(conn.get_all_autoscaling_instances())
    instances.should.have.length_of(2)

    conn.set_desired_capacity("tester_group", 1)
    group = conn.get_all_groups()[0]
    group.desired_capacity.should.equal(1)

    instances = list(conn.get_all_autoscaling_instances())
    instances.should.have.length_of(1)


@requires_boto_gte("2.8")
@mock_autoscaling_deprecated
def test_set_desired_capacity_the_same():
    mocked_networking = setup_networking_deprecated()
    conn = boto.connect_autoscale()
    config = LaunchConfiguration(
        name='tester',
        image_id='ami-abcd1234',
        instance_type='t2.medium',
    )
    conn.create_launch_configuration(config)

    group = AutoScalingGroup(
        name='tester_group',
        availability_zones=['us-east-1a'],
        desired_capacity=2,
        max_size=2,
        min_size=2,
        launch_config=config,
        vpc_zone_identifier=mocked_networking['subnet1'],
    )
    conn.create_auto_scaling_group(group)

    group = conn.get_all_groups()[0]
    group.desired_capacity.should.equal(2)
    instances = list(conn.get_all_autoscaling_instances())
    instances.should.have.length_of(2)

    conn.set_desired_capacity("tester_group", 2)
    group = conn.get_all_groups()[0]
    group.desired_capacity.should.equal(2)

    instances = list(conn.get_all_autoscaling_instances())
    instances.should.have.length_of(2)


@mock_autoscaling_deprecated
@mock_elb_deprecated
def test_autoscaling_group_with_elb():
    mocked_networking = setup_networking_deprecated()
    elb_conn = boto.connect_elb()
    zones = ['us-east-1a', 'us-east-1b']
    ports = [(80, 8080, 'http'), (443, 8443, 'tcp')]
    lb = elb_conn.create_load_balancer('my-lb', zones, ports)
    instances_health = elb_conn.describe_instance_health('my-lb')
    instances_health.should.be.empty

    conn = boto.connect_autoscale()
    config = LaunchConfiguration(
        name='tester',
        image_id='ami-abcd1234',
        instance_type='t2.medium',
    )
    conn.create_launch_configuration(config)
    group = AutoScalingGroup(
        name='tester_group',
        max_size=2,
        min_size=2,
        launch_config=config,
        load_balancers=["my-lb"],
        vpc_zone_identifier=mocked_networking['subnet1'],
    )
    conn.create_auto_scaling_group(group)
    group = conn.get_all_groups()[0]
    elb = elb_conn.get_all_load_balancers()[0]
    group.desired_capacity.should.equal(2)
    elb.instances.should.have.length_of(2)

    autoscale_instance_ids = set(
        instance.instance_id for instance in group.instances)
    elb_instace_ids = set(instance.id for instance in elb.instances)
    autoscale_instance_ids.should.equal(elb_instace_ids)

    conn.set_desired_capacity("tester_group", 3)
    group = conn.get_all_groups()[0]
    elb = elb_conn.get_all_load_balancers()[0]
    group.desired_capacity.should.equal(3)
    elb.instances.should.have.length_of(3)

    autoscale_instance_ids = set(
        instance.instance_id for instance in group.instances)
    elb_instace_ids = set(instance.id for instance in elb.instances)
    autoscale_instance_ids.should.equal(elb_instace_ids)

    conn.delete_auto_scaling_group('tester_group')
    conn.get_all_groups().should.have.length_of(0)
    elb = elb_conn.get_all_load_balancers()[0]
    elb.instances.should.have.length_of(0)


'''
Boto3
'''


@mock_autoscaling
@mock_elb
def test_describe_load_balancers():
    mocked_networking = setup_networking()
    INSTANCE_COUNT = 2

    elb_client = boto3.client('elb', region_name='us-east-1')
    elb_client.create_load_balancer(
        LoadBalancerName='my-lb',
        Listeners=[
            {'Protocol': 'tcp', 'LoadBalancerPort': 80, 'InstancePort': 8080}],
        AvailabilityZones=['us-east-1a', 'us-east-1b']
    )

    client = boto3.client('autoscaling', region_name='us-east-1')
    client.create_launch_configuration(
        LaunchConfigurationName='test_launch_configuration'
    )
    client.create_auto_scaling_group(
        AutoScalingGroupName='test_asg',
        LaunchConfigurationName='test_launch_configuration',
        LoadBalancerNames=['my-lb'],
        MinSize=0,
        MaxSize=INSTANCE_COUNT,
        DesiredCapacity=INSTANCE_COUNT,
        Tags=[{
            "ResourceId": 'test_asg',
            "Key": 'test_key',
            "Value": 'test_value',
            "PropagateAtLaunch": True
        }],
        VPCZoneIdentifier=mocked_networking['subnet1'],
    )

    response = client.describe_load_balancers(AutoScalingGroupName='test_asg')
    assert response['ResponseMetadata']['RequestId']
    list(response['LoadBalancers']).should.have.length_of(1)
    response['LoadBalancers'][0]['LoadBalancerName'].should.equal('my-lb')

@mock_autoscaling
@mock_elb
def test_create_elb_and_autoscaling_group_no_relationship():
    mocked_networking = setup_networking()
    INSTANCE_COUNT = 2
    ELB_NAME = 'my-elb'

    elb_client = boto3.client('elb', region_name='us-east-1')
    elb_client.create_load_balancer(
        LoadBalancerName=ELB_NAME,
        Listeners=[
            {'Protocol': 'tcp', 'LoadBalancerPort': 80, 'InstancePort': 8080}],
        AvailabilityZones=['us-east-1a', 'us-east-1b']
    )

    client = boto3.client('autoscaling', region_name='us-east-1')
    client.create_launch_configuration(
        LaunchConfigurationName='test_launch_configuration'
    )

    client.create_auto_scaling_group(
        AutoScalingGroupName='test_asg',
        LaunchConfigurationName='test_launch_configuration',
        MinSize=0,
        MaxSize=INSTANCE_COUNT,
        DesiredCapacity=INSTANCE_COUNT,
        VPCZoneIdentifier=mocked_networking['subnet1'],
    )

    # autoscaling group and elb should have no relationship
    response = client.describe_load_balancers(
        AutoScalingGroupName='test_asg'
    )
    list(response['LoadBalancers']).should.have.length_of(0)
    response = elb_client.describe_load_balancers(
        LoadBalancerNames=[ELB_NAME]
    )
    list(response['LoadBalancerDescriptions'][0]['Instances']).should.have.length_of(0)


@mock_autoscaling
@mock_elb
def test_attach_load_balancer():
    mocked_networking = setup_networking()
    INSTANCE_COUNT = 2

    elb_client = boto3.client('elb', region_name='us-east-1')
    elb_client.create_load_balancer(
        LoadBalancerName='my-lb',
        Listeners=[
            {'Protocol': 'tcp', 'LoadBalancerPort': 80, 'InstancePort': 8080}],
        AvailabilityZones=['us-east-1a', 'us-east-1b']
    )

    client = boto3.client('autoscaling', region_name='us-east-1')
    client.create_launch_configuration(
        LaunchConfigurationName='test_launch_configuration'
    )
    client.create_auto_scaling_group(
        AutoScalingGroupName='test_asg',
        LaunchConfigurationName='test_launch_configuration',
        MinSize=0,
        MaxSize=INSTANCE_COUNT,
        DesiredCapacity=INSTANCE_COUNT,
        Tags=[{
            "ResourceId": 'test_asg',
            "Key": 'test_key',
            "Value": 'test_value',
            "PropagateAtLaunch": True
        }],
        VPCZoneIdentifier=mocked_networking['subnet1'],
    )

    response = client.attach_load_balancers(
        AutoScalingGroupName='test_asg',
        LoadBalancerNames=['my-lb'])
    response['ResponseMetadata']['HTTPStatusCode'].should.equal(200)

    response = elb_client.describe_load_balancers(
        LoadBalancerNames=['my-lb']
    )
    list(response['LoadBalancerDescriptions'][0]['Instances']).should.have.length_of(INSTANCE_COUNT)

    response = client.describe_auto_scaling_groups(
        AutoScalingGroupNames=["test_asg"]
    )
    list(response['AutoScalingGroups'][0]['LoadBalancerNames']).should.have.length_of(1)


@mock_autoscaling
@mock_elb
def test_detach_load_balancer():
    mocked_networking = setup_networking()
    INSTANCE_COUNT = 2

    elb_client = boto3.client('elb', region_name='us-east-1')
    elb_client.create_load_balancer(
        LoadBalancerName='my-lb',
        Listeners=[
            {'Protocol': 'tcp', 'LoadBalancerPort': 80, 'InstancePort': 8080}],
        AvailabilityZones=['us-east-1a', 'us-east-1b']
    )

    client = boto3.client('autoscaling', region_name='us-east-1')
    client.create_launch_configuration(
        LaunchConfigurationName='test_launch_configuration'
    )
    client.create_auto_scaling_group(
        AutoScalingGroupName='test_asg',
        LaunchConfigurationName='test_launch_configuration',
        LoadBalancerNames=['my-lb'],
        MinSize=0,
        MaxSize=INSTANCE_COUNT,
        DesiredCapacity=INSTANCE_COUNT,
        Tags=[{
            "ResourceId": 'test_asg',
            "Key": 'test_key',
            "Value": 'test_value',
            "PropagateAtLaunch": True
        }],
        VPCZoneIdentifier=mocked_networking['subnet1'],
    )

    response = client.detach_load_balancers(
        AutoScalingGroupName='test_asg',
        LoadBalancerNames=['my-lb'])
    response['ResponseMetadata']['HTTPStatusCode'].should.equal(200)

    response = elb_client.describe_load_balancers(
        LoadBalancerNames=['my-lb']
    )
    list(response['LoadBalancerDescriptions'][0]['Instances']).should.have.length_of(0)

    response = client.describe_load_balancers(AutoScalingGroupName='test_asg')
    list(response['LoadBalancers']).should.have.length_of(0)


@mock_autoscaling
def test_create_autoscaling_group_boto3():
    mocked_networking = setup_networking()
    client = boto3.client('autoscaling', region_name='us-east-1')
    _ = client.create_launch_configuration(
        LaunchConfigurationName='test_launch_configuration'
    )
    response = client.create_auto_scaling_group(
        AutoScalingGroupName='test_asg',
        LaunchConfigurationName='test_launch_configuration',
        MinSize=0,
        MaxSize=20,
        DesiredCapacity=5,
        Tags=[
            {'ResourceId': 'test_asg',
             'ResourceType': 'auto-scaling-group',
             'Key': 'propogated-tag-key',
             'Value': 'propogate-tag-value',
             'PropagateAtLaunch': True
             },
            {'ResourceId': 'test_asg',
             'ResourceType': 'auto-scaling-group',
             'Key': 'not-propogated-tag-key',
             'Value': 'not-propogate-tag-value',
             'PropagateAtLaunch': False
             }],
        VPCZoneIdentifier=mocked_networking['subnet1'],
        NewInstancesProtectedFromScaleIn=False,
    )
    response['ResponseMetadata']['HTTPStatusCode'].should.equal(200)


@mock_autoscaling
def test_describe_autoscaling_groups_boto3():
    mocked_networking = setup_networking()
    client = boto3.client('autoscaling', region_name='us-east-1')
    _ = client.create_launch_configuration(
        LaunchConfigurationName='test_launch_configuration'
    )
    _ = client.create_auto_scaling_group(
        AutoScalingGroupName='test_asg',
        LaunchConfigurationName='test_launch_configuration',
        MinSize=0,
        MaxSize=20,
        DesiredCapacity=5,
        VPCZoneIdentifier=mocked_networking['subnet1'],
        NewInstancesProtectedFromScaleIn=True,
    )

    response = client.describe_auto_scaling_groups(
        AutoScalingGroupNames=["test_asg"]
    )
    response['ResponseMetadata']['HTTPStatusCode'].should.equal(200)
    group = response['AutoScalingGroups'][0]
    group['AutoScalingGroupName'].should.equal('test_asg')
    group['AvailabilityZones'].should.equal(['us-east-1a'])
    group['VPCZoneIdentifier'].should.equal(mocked_networking['subnet1'])
    group['NewInstancesProtectedFromScaleIn'].should.equal(True)
    for instance in group['Instances']:
        instance['AvailabilityZone'].should.equal('us-east-1a')
        instance['ProtectedFromScaleIn'].should.equal(True)


@mock_autoscaling
def test_describe_autoscaling_instances_boto3():
    mocked_networking = setup_networking()
    client = boto3.client('autoscaling', region_name='us-east-1')
    _ = client.create_launch_configuration(
        LaunchConfigurationName='test_launch_configuration'
    )
    _ = client.create_auto_scaling_group(
        AutoScalingGroupName='test_asg',
        LaunchConfigurationName='test_launch_configuration',
        MinSize=0,
        MaxSize=20,
        DesiredCapacity=5,
        VPCZoneIdentifier=mocked_networking['subnet1'],
        NewInstancesProtectedFromScaleIn=True,
    )

    response = client.describe_auto_scaling_groups(
        AutoScalingGroupNames=["test_asg"]
    )
    instance_ids = [
        instance['InstanceId']
        for instance in response['AutoScalingGroups'][0]['Instances']
    ]

    response = client.describe_auto_scaling_instances(InstanceIds=instance_ids)
    for instance in response['AutoScalingInstances']:
        instance['AutoScalingGroupName'].should.equal('test_asg')
        instance['AvailabilityZone'].should.equal('us-east-1a')
        instance['ProtectedFromScaleIn'].should.equal(True)


@mock_autoscaling
def test_update_autoscaling_group_boto3():
    mocked_networking = setup_networking()
    client = boto3.client('autoscaling', region_name='us-east-1')
    _ = client.create_launch_configuration(
        LaunchConfigurationName='test_launch_configuration'
    )
    _ = client.create_auto_scaling_group(
        AutoScalingGroupName='test_asg',
        LaunchConfigurationName='test_launch_configuration',
        MinSize=0,
        MaxSize=20,
        DesiredCapacity=5,
        VPCZoneIdentifier=mocked_networking['subnet1'],
        NewInstancesProtectedFromScaleIn=True,
    )

    _ = client.update_auto_scaling_group(
        AutoScalingGroupName='test_asg',
        MinSize=1,
        VPCZoneIdentifier="{subnet1},{subnet2}".format(
            subnet1=mocked_networking['subnet1'],
            subnet2=mocked_networking['subnet2'],
        ),
        NewInstancesProtectedFromScaleIn=False,
    )

    response = client.describe_auto_scaling_groups(
        AutoScalingGroupNames=["test_asg"]
    )
    group = response['AutoScalingGroups'][0]
    group['MinSize'].should.equal(1)
    set(group['AvailabilityZones']).should.equal({'us-east-1a', 'us-east-1b'})
    group['NewInstancesProtectedFromScaleIn'].should.equal(False)


@mock_autoscaling
def test_autoscaling_taqs_update_boto3():
    mocked_networking = setup_networking()
    client = boto3.client('autoscaling', region_name='us-east-1')
    _ = client.create_launch_configuration(
        LaunchConfigurationName='test_launch_configuration'
    )
    _ = client.create_auto_scaling_group(
        AutoScalingGroupName='test_asg',
        LaunchConfigurationName='test_launch_configuration',
        MinSize=0,
        MaxSize=20,
        DesiredCapacity=5,
        Tags=[{
            "ResourceId": 'test_asg',
            "Key": 'test_key',
            "Value": 'test_value',
            "PropagateAtLaunch": True
        }],
        VPCZoneIdentifier=mocked_networking['subnet1'],
    )

    client.create_or_update_tags(Tags=[{
        "ResourceId": 'test_asg',
        "Key": 'test_key',
        "Value": 'updated_test_value',
        "PropagateAtLaunch": True
    }, {
        "ResourceId": 'test_asg',
        "Key": 'test_key2',
        "Value": 'test_value2',
        "PropagateAtLaunch": False
    }])

    response = client.describe_auto_scaling_groups(
        AutoScalingGroupNames=["test_asg"]
    )
    response['AutoScalingGroups'][0]['Tags'].should.have.length_of(2)


@mock_autoscaling
def test_autoscaling_describe_policies_boto3():
    mocked_networking = setup_networking()
    client = boto3.client('autoscaling', region_name='us-east-1')
    _ = client.create_launch_configuration(
        LaunchConfigurationName='test_launch_configuration'
    )
    _ = client.create_auto_scaling_group(
        AutoScalingGroupName='test_asg',
        LaunchConfigurationName='test_launch_configuration',
        MinSize=0,
        MaxSize=20,
        DesiredCapacity=5,
        Tags=[{
            "ResourceId": 'test_asg',
            "Key": 'test_key',
            "Value": 'test_value',
            "PropagateAtLaunch": True
        }],
        VPCZoneIdentifier=mocked_networking['subnet1'],
    )

    client.put_scaling_policy(
        AutoScalingGroupName='test_asg',
        PolicyName='test_policy_down',
        PolicyType='SimpleScaling',
        AdjustmentType='PercentChangeInCapacity',
        ScalingAdjustment=-10,
        Cooldown=60,
        MinAdjustmentMagnitude=1)
    client.put_scaling_policy(
        AutoScalingGroupName='test_asg',
        PolicyName='test_policy_up',
        PolicyType='SimpleScaling',
        AdjustmentType='PercentChangeInCapacity',
        ScalingAdjustment=10,
        Cooldown=60,
        MinAdjustmentMagnitude=1)

    response = client.describe_policies()
    response['ScalingPolicies'].should.have.length_of(2)

    response = client.describe_policies(AutoScalingGroupName='test_asg')
    response['ScalingPolicies'].should.have.length_of(2)

    response = client.describe_policies(PolicyTypes=['StepScaling'])
    response['ScalingPolicies'].should.have.length_of(0)

    response = client.describe_policies(
        AutoScalingGroupName='test_asg',
        PolicyNames=['test_policy_down'],
        PolicyTypes=['SimpleScaling']
    )
    response['ScalingPolicies'].should.have.length_of(1)
    response['ScalingPolicies'][0][
        'PolicyName'].should.equal('test_policy_down')

@mock_autoscaling
@mock_ec2
def test_detach_one_instance_decrement():
    mocked_networking = setup_networking()
    client = boto3.client('autoscaling', region_name='us-east-1')
    _ = client.create_launch_configuration(
        LaunchConfigurationName='test_launch_configuration'
    )
    client.create_auto_scaling_group(
        AutoScalingGroupName='test_asg',
        LaunchConfigurationName='test_launch_configuration',
        MinSize=0,
        MaxSize=2,
        DesiredCapacity=2,
        Tags=[{
            'ResourceId': 'test_asg',
            'ResourceType': 'auto-scaling-group',
            'Key': 'propogated-tag-key',
            'Value': 'propogate-tag-value',
            'PropagateAtLaunch': True
        }],
        VPCZoneIdentifier=mocked_networking['subnet1'],
    )
    response = client.describe_auto_scaling_groups(
        AutoScalingGroupNames=['test_asg']
    )
    instance_to_detach = response['AutoScalingGroups'][0]['Instances'][0]['InstanceId']
    instance_to_keep = response['AutoScalingGroups'][0]['Instances'][1]['InstanceId']

    ec2_client = boto3.client('ec2', region_name='us-east-1')

    response = ec2_client.describe_instances(InstanceIds=[instance_to_detach])

    response = client.detach_instances(
        AutoScalingGroupName='test_asg',
        InstanceIds=[instance_to_detach],
        ShouldDecrementDesiredCapacity=True
    )
    response['ResponseMetadata']['HTTPStatusCode'].should.equal(200)

    response = client.describe_auto_scaling_groups(
        AutoScalingGroupNames=['test_asg']
    )
    response['AutoScalingGroups'][0]['Instances'].should.have.length_of(1)

    # test to ensure tag has been removed
    response = ec2_client.describe_instances(InstanceIds=[instance_to_detach])
    tags = response['Reservations'][0]['Instances'][0]['Tags']
    tags.should.have.length_of(1)

    # test to ensure tag is present on other instance
    response = ec2_client.describe_instances(InstanceIds=[instance_to_keep])
    tags = response['Reservations'][0]['Instances'][0]['Tags']
    tags.should.have.length_of(2)

@mock_autoscaling
@mock_ec2
def test_detach_one_instance():
    mocked_networking = setup_networking()
    client = boto3.client('autoscaling', region_name='us-east-1')
    _ = client.create_launch_configuration(
        LaunchConfigurationName='test_launch_configuration'
    )
    client.create_auto_scaling_group(
        AutoScalingGroupName='test_asg',
        LaunchConfigurationName='test_launch_configuration',
        MinSize=0,
        MaxSize=2,
        DesiredCapacity=2,
        Tags=[{
            'ResourceId': 'test_asg',
            'ResourceType': 'auto-scaling-group',
            'Key': 'propogated-tag-key',
            'Value': 'propogate-tag-value',
            'PropagateAtLaunch': True
        }],
        VPCZoneIdentifier=mocked_networking['subnet1'],
    )
    response = client.describe_auto_scaling_groups(
        AutoScalingGroupNames=['test_asg']
    )
    instance_to_detach = response['AutoScalingGroups'][0]['Instances'][0]['InstanceId']
    instance_to_keep = response['AutoScalingGroups'][0]['Instances'][1]['InstanceId']

    ec2_client = boto3.client('ec2', region_name='us-east-1')

    response = ec2_client.describe_instances(InstanceIds=[instance_to_detach])

    response = client.detach_instances(
        AutoScalingGroupName='test_asg',
        InstanceIds=[instance_to_detach],
        ShouldDecrementDesiredCapacity=False
    )
    response['ResponseMetadata']['HTTPStatusCode'].should.equal(200)

    response = client.describe_auto_scaling_groups(
        AutoScalingGroupNames=['test_asg']
    )
    # test to ensure instance was replaced
    response['AutoScalingGroups'][0]['Instances'].should.have.length_of(2)

    response = ec2_client.describe_instances(InstanceIds=[instance_to_detach])
    tags = response['Reservations'][0]['Instances'][0]['Tags']
    tags.should.have.length_of(1)

    response = ec2_client.describe_instances(InstanceIds=[instance_to_keep])
    tags = response['Reservations'][0]['Instances'][0]['Tags']
    tags.should.have.length_of(2)

@mock_autoscaling
@mock_ec2
def test_attach_one_instance():
    mocked_networking = setup_networking()
    client = boto3.client('autoscaling', region_name='us-east-1')
    _ = client.create_launch_configuration(
        LaunchConfigurationName='test_launch_configuration'
    )
    client.create_auto_scaling_group(
        AutoScalingGroupName='test_asg',
        LaunchConfigurationName='test_launch_configuration',
        MinSize=0,
        MaxSize=4,
        DesiredCapacity=2,
        Tags=[{
            'ResourceId': 'test_asg',
            'ResourceType': 'auto-scaling-group',
            'Key': 'propogated-tag-key',
            'Value': 'propogate-tag-value',
            'PropagateAtLaunch': True
        }],
        VPCZoneIdentifier=mocked_networking['subnet1'],
        NewInstancesProtectedFromScaleIn=True,
    )

    ec2 = boto3.resource('ec2', 'us-east-1')
    instances_to_add = [x.id for x in ec2.create_instances(ImageId='', MinCount=1, MaxCount=1)]

    response = client.attach_instances(
        AutoScalingGroupName='test_asg',
        InstanceIds=instances_to_add
    )
    response['ResponseMetadata']['HTTPStatusCode'].should.equal(200)

    response = client.describe_auto_scaling_groups(
        AutoScalingGroupNames=['test_asg']
    )
    instances = response['AutoScalingGroups'][0]['Instances']
    instances.should.have.length_of(3)
    for instance in instances:
        instance['ProtectedFromScaleIn'].should.equal(True)


@mock_autoscaling
@mock_ec2
def test_describe_instance_health():
    mocked_networking = setup_networking()
    client = boto3.client('autoscaling', region_name='us-east-1')
    _ = client.create_launch_configuration(
        LaunchConfigurationName='test_launch_configuration'
    )
    client.create_auto_scaling_group(
        AutoScalingGroupName='test_asg',
        LaunchConfigurationName='test_launch_configuration',
        MinSize=2,
        MaxSize=4,
        DesiredCapacity=2,
        VPCZoneIdentifier=mocked_networking['subnet1'],
    )

    response = client.describe_auto_scaling_groups(
        AutoScalingGroupNames=['test_asg']
    )

    instance1 = response['AutoScalingGroups'][0]['Instances'][0]
    instance1['HealthStatus'].should.equal('Healthy')

@mock_autoscaling
@mock_ec2
def test_set_instance_health():
    mocked_networking = setup_networking()
    client = boto3.client('autoscaling', region_name='us-east-1')
    _ = client.create_launch_configuration(
        LaunchConfigurationName='test_launch_configuration'
    )
    client.create_auto_scaling_group(
        AutoScalingGroupName='test_asg',
        LaunchConfigurationName='test_launch_configuration',
        MinSize=2,
        MaxSize=4,
        DesiredCapacity=2,
        VPCZoneIdentifier=mocked_networking['subnet1'],
    )

    response = client.describe_auto_scaling_groups(
        AutoScalingGroupNames=['test_asg']
    )

    instance1 = response['AutoScalingGroups'][0]['Instances'][0]
    instance1['HealthStatus'].should.equal('Healthy')

    client.set_instance_health(InstanceId=instance1['InstanceId'], HealthStatus='Unhealthy')

    response = client.describe_auto_scaling_groups(
        AutoScalingGroupNames=['test_asg']
    )

    instance1 = response['AutoScalingGroups'][0]['Instances'][0]
    instance1['HealthStatus'].should.equal('Unhealthy')

@mock_autoscaling
def test_suspend_processes():
    mocked_networking = setup_networking()
    client = boto3.client('autoscaling', region_name='us-east-1')
    client.create_launch_configuration(
        LaunchConfigurationName='lc',
    )
    client.create_auto_scaling_group(
        LaunchConfigurationName='lc',
        AutoScalingGroupName='test-asg',
        MinSize=1,
        MaxSize=1,
        VPCZoneIdentifier=mocked_networking['subnet1'],
    )

    # When we suspend the 'Launch' process on the ASG client
    client.suspend_processes(
        AutoScalingGroupName='test-asg',
        ScalingProcesses=['Launch']
    )

    res = client.describe_auto_scaling_groups(
        AutoScalingGroupNames=['test-asg']
    )

    # The 'Launch' process should, in fact, be suspended
    launch_suspended = False
    for proc in res['AutoScalingGroups'][0]['SuspendedProcesses']:
        if proc.get('ProcessName') == 'Launch':
            launch_suspended = True

    assert launch_suspended is True

@mock_autoscaling
def test_set_instance_protection():
    mocked_networking = setup_networking()
    client = boto3.client('autoscaling', region_name='us-east-1')
    _ = client.create_launch_configuration(
        LaunchConfigurationName='test_launch_configuration'
    )
    _ = client.create_auto_scaling_group(
        AutoScalingGroupName='test_asg',
        LaunchConfigurationName='test_launch_configuration',
        MinSize=0,
        MaxSize=20,
        DesiredCapacity=5,
        VPCZoneIdentifier=mocked_networking['subnet1'],
        NewInstancesProtectedFromScaleIn=False,
    )

    response = client.describe_auto_scaling_groups(AutoScalingGroupNames=['test_asg'])
    instance_ids = [
        instance['InstanceId']
        for instance in response['AutoScalingGroups'][0]['Instances']
    ]
    protected = instance_ids[:3]

    _ = client.set_instance_protection(
        AutoScalingGroupName='test_asg',
        InstanceIds=protected,
        ProtectedFromScaleIn=True,
    )

    response = client.describe_auto_scaling_groups(AutoScalingGroupNames=['test_asg'])
    for instance in response['AutoScalingGroups'][0]['Instances']:
        instance['ProtectedFromScaleIn'].should.equal(
            instance['InstanceId'] in protected
        )


@mock_autoscaling
def test_set_desired_capacity_up_boto3():
    mocked_networking = setup_networking()
    client = boto3.client('autoscaling', region_name='us-east-1')
    _ = client.create_launch_configuration(
        LaunchConfigurationName='test_launch_configuration'
    )
    _ = client.create_auto_scaling_group(
        AutoScalingGroupName='test_asg',
        LaunchConfigurationName='test_launch_configuration',
        MinSize=0,
        MaxSize=20,
        DesiredCapacity=5,
        VPCZoneIdentifier=mocked_networking['subnet1'],
        NewInstancesProtectedFromScaleIn=True,
    )

    _ = client.set_desired_capacity(
        AutoScalingGroupName='test_asg',
        DesiredCapacity=10,
    )

    response = client.describe_auto_scaling_groups(AutoScalingGroupNames=['test_asg'])
    instances = response['AutoScalingGroups'][0]['Instances']
    instances.should.have.length_of(10)
    for instance in instances:
        instance['ProtectedFromScaleIn'].should.equal(True)


@mock_autoscaling
def test_set_desired_capacity_down_boto3():
    mocked_networking = setup_networking()
    client = boto3.client('autoscaling', region_name='us-east-1')
    _ = client.create_launch_configuration(
        LaunchConfigurationName='test_launch_configuration'
    )
    _ = client.create_auto_scaling_group(
        AutoScalingGroupName='test_asg',
        LaunchConfigurationName='test_launch_configuration',
        MinSize=0,
        MaxSize=20,
        DesiredCapacity=5,
        VPCZoneIdentifier=mocked_networking['subnet1'],
        NewInstancesProtectedFromScaleIn=True,
    )

    response = client.describe_auto_scaling_groups(AutoScalingGroupNames=['test_asg'])
    instance_ids = [
        instance['InstanceId']
        for instance in response['AutoScalingGroups'][0]['Instances']
    ]
    unprotected, protected = instance_ids[:2], instance_ids[2:]

    _ = client.set_instance_protection(
        AutoScalingGroupName='test_asg',
        InstanceIds=unprotected,
        ProtectedFromScaleIn=False,
    )

    _ = client.set_desired_capacity(
        AutoScalingGroupName='test_asg',
        DesiredCapacity=1,
    )

    response = client.describe_auto_scaling_groups(AutoScalingGroupNames=['test_asg'])
    group = response['AutoScalingGroups'][0]
    group['DesiredCapacity'].should.equal(1)
    instance_ids = {instance['InstanceId'] for instance in group['Instances']}
    set(protected).should.equal(instance_ids)
    set(unprotected).should_not.be.within(instance_ids)  # only unprotected killed



import os
import re
import json

from moto.core.exceptions import JsonRESTError
from moto.core import BaseBackend, BaseModel


class Rule(BaseModel):

    def _generate_arn(self, name):
        return 'arn:aws:events:us-west-2:111111111111:rule/' + name

    def __init__(self, name, **kwargs):
        self.name = name
        self.arn = kwargs.get('Arn') or self._generate_arn(name)
        self.event_pattern = kwargs.get('EventPattern')
        self.schedule_exp = kwargs.get('ScheduleExpression')
        self.state = kwargs.get('State') or 'ENABLED'
        self.description = kwargs.get('Description')
        self.role_arn = kwargs.get('RoleArn')
        self.targets = []

    def enable(self):
        self.state = 'ENABLED'

    def disable(self):
        self.state = 'DISABLED'

    # This song and dance for targets is because we need order for Limits and NextTokens, but can't use OrderedDicts
    # with Python 2.6, so tracking it with an array it is.
    def _check_target_exists(self, target_id):
        for i in range(0, len(self.targets)):
            if target_id == self.targets[i]['Id']:
                return i
        return None

    def put_targets(self, targets):
        # Not testing for valid ARNs.
        for target in targets:
            index = self._check_target_exists(target['Id'])
            if index is not None:
                self.targets[index] = target
            else:
                self.targets.append(target)

    def remove_targets(self, ids):
        for target_id in ids:
            index = self._check_target_exists(target_id)
            if index is not None:
                self.targets.pop(index)


class EventsBackend(BaseBackend):
    ACCOUNT_ID = re.compile(r'^(\d{1,12}|\*)$')
    STATEMENT_ID = re.compile(r'^[a-zA-Z0-9-_]{1,64}$')

    def __init__(self):
        self.rules = {}
        # This array tracks the order in which the rules have been added, since
        # 2.6 doesn't have OrderedDicts.
        self.rules_order = []
        self.next_tokens = {}

        self.permissions = {}

    def _get_rule_by_index(self, i):
        return self.rules.get(self.rules_order[i])

    def _gen_next_token(self, index):
        token = os.urandom(128).encode('base64')
        self.next_tokens[token] = index
        return token

    def _process_token_and_limits(self, array_len, next_token=None, limit=None):
        start_index = 0
        end_index = array_len
        new_next_token = None

        if next_token:
            start_index = self.next_tokens.pop(next_token, 0)

        if limit is not None:
            new_end_index = start_index + int(limit)
            if new_end_index < end_index:
                end_index = new_end_index
                new_next_token = self._gen_next_token(end_index)

        return start_index, end_index, new_next_token

    def delete_rule(self, name):
        self.rules_order.pop(self.rules_order.index(name))
        return self.rules.pop(name) is not None

    def describe_rule(self, name):
        return self.rules.get(name)

    def disable_rule(self, name):
        if name in self.rules:
            self.rules[name].disable()
            return True

        return False

    def enable_rule(self, name):
        if name in self.rules:
            self.rules[name].enable()
            return True

        return False

    def list_rule_names_by_target(self, target_arn, next_token=None, limit=None):
        matching_rules = []
        return_obj = {}

        start_index, end_index, new_next_token = self._process_token_and_limits(
            len(self.rules), next_token, limit)

        for i in range(start_index, end_index):
            rule = self._get_rule_by_index(i)
            for target in rule.targets:
                if target['Arn'] == target_arn:
                    matching_rules.append(rule.name)

        return_obj['RuleNames'] = matching_rules
        if new_next_token is not None:
            return_obj['NextToken'] = new_next_token

        return return_obj

    def list_rules(self, prefix=None, next_token=None, limit=None):
        match_string = '.*'
        if prefix is not None:
            match_string = '^' + prefix + match_string

        match_regex = re.compile(match_string)

        matching_rules = []
        return_obj = {}

        start_index, end_index, new_next_token = self._process_token_and_limits(
            len(self.rules), next_token, limit)

        for i in range(start_index, end_index):
            rule = self._get_rule_by_index(i)
            if match_regex.match(rule.name):
                matching_rules.append(rule)

        return_obj['Rules'] = matching_rules
        if new_next_token is not None:
            return_obj['NextToken'] = new_next_token

        return return_obj

    def list_targets_by_rule(self, rule, next_token=None, limit=None):
        # We'll let a KeyError exception be thrown for response to handle if
        # rule doesn't exist.
        rule = self.rules[rule]

        start_index, end_index, new_next_token = self._process_token_and_limits(
            len(rule.targets), next_token, limit)

        returned_targets = []
        return_obj = {}

        for i in range(start_index, end_index):
            returned_targets.append(rule.targets[i])

        return_obj['Targets'] = returned_targets
        if new_next_token is not None:
            return_obj['NextToken'] = new_next_token

        return return_obj

    def put_rule(self, name, **kwargs):
        rule = Rule(name, **kwargs)
        self.rules[rule.name] = rule
        self.rules_order.append(rule.name)
        return rule.arn

    def put_targets(self, name, targets):
        rule = self.rules.get(name)

        if rule:
            rule.put_targets(targets)
            return True

        return False

    def put_events(self, events):
        num_events = len(events)

        if num_events < 1:
            raise JsonRESTError('ValidationError', 'Need at least 1 event')
        elif num_events > 10:
            raise JsonRESTError('ValidationError', 'Can only submit 10 events at once')

        # We dont really need to store the events yet
        return []

    def remove_targets(self, name, ids):
        rule = self.rules.get(name)

        if rule:
            rule.remove_targets(ids)
            return True

        return False

    def test_event_pattern(self):
        raise NotImplementedError()

    def put_permission(self, action, principal, statement_id):
        if action is None or action != 'events:PutEvents':
            raise JsonRESTError('InvalidParameterValue', 'Action must be PutEvents')

        if principal is None or self.ACCOUNT_ID.match(principal) is None:
            raise JsonRESTError('InvalidParameterValue', 'Principal must match ^(\d{1,12}|\*)$')

        if statement_id is None or self.STATEMENT_ID.match(statement_id) is None:
            raise JsonRESTError('InvalidParameterValue', 'StatementId must match ^[a-zA-Z0-9-_]{1,64}$')

        self.permissions[statement_id] = {'action': action, 'principal': principal}

    def remove_permission(self, statement_id):
        try:
            del self.permissions[statement_id]
        except KeyError:
            raise JsonRESTError('ResourceNotFoundException', 'StatementId not found')

    def describe_event_bus(self):
        arn = "arn:aws:events:us-east-1:000000000000:event-bus/default"
        statements = []
        for statement_id, data in self.permissions.items():
            statements.append({
                'Sid': statement_id,
                'Effect': 'Allow',
                'Principal': {'AWS': 'arn:aws:iam::{0}:root'.format(data['principal'])},
                'Action': data['action'],
                'Resource': arn
            })
        policy = {'Version': '2012-10-17', 'Statement': statements}
        policy_json = json.dumps(policy)
        return {
            'Policy': policy_json,
            'Name': 'default',
            'Arn': arn
        }


events_backend = EventsBackend()



from __future__ import unicode_literals
import sure  # noqa

import moto.server as server

'''
Test the different server responses
'''


def test_sts_get_session_token():
    backend = server.create_backend_app("sts")
    test_client = backend.test_client()

    res = test_client.get('/?Action=GetSessionToken')
    res.status_code.should.equal(200)
    res.data.should.contain(b"SessionToken")
    res.data.should.contain(b"AccessKeyId")


def test_sts_get_federation_token():
    backend = server.create_backend_app("sts")
    test_client = backend.test_client()

    res = test_client.get('/?Action=GetFederationToken&Name=Bob')
    res.status_code.should.equal(200)
    res.data.should.contain(b"SessionToken")
    res.data.should.contain(b"AccessKeyId")


def test_sts_get_caller_identity():
    backend = server.create_backend_app("sts")
    test_client = backend.test_client()

    res = test_client.get('/?Action=GetCallerIdentity')
    res.status_code.should.equal(200)
    res.data.should.contain(b"Arn")
    res.data.should.contain(b"UserId")
    res.data.should.contain(b"Account")



from __future__ import unicode_literals
from moto.core.responses import BaseResponse


class Windows(BaseResponse):

    def bundle_instance(self):
        raise NotImplementedError(
            'Windows.bundle_instance is not yet implemented')

    def cancel_bundle_task(self):
        raise NotImplementedError(
            'Windows.cancel_bundle_task is not yet implemented')

    def describe_bundle_tasks(self):
        raise NotImplementedError(
            'Windows.describe_bundle_tasks is not yet implemented')

    def get_password_data(self):
        raise NotImplementedError(
            'Windows.get_password_data is not yet implemented')



from __future__ import unicode_literals

import base64
import json
import re
import six

from moto.core.responses import BaseResponse
from .models import kms_backends
from .exceptions import NotFoundException, ValidationException, AlreadyExistsException, NotAuthorizedException

reserved_aliases = [
    'alias/aws/ebs',
    'alias/aws/s3',
    'alias/aws/redshift',
    'alias/aws/rds',
]


class KmsResponse(BaseResponse):

    @property
    def parameters(self):
        return json.loads(self.body)

    @property
    def kms_backend(self):
        return kms_backends[self.region]

    def create_key(self):
        policy = self.parameters.get('Policy')
        key_usage = self.parameters.get('KeyUsage')
        description = self.parameters.get('Description')

        key = self.kms_backend.create_key(
            policy, key_usage, description, self.region)
        return json.dumps(key.to_dict())

    def update_key_description(self):
        key_id = self.parameters.get('KeyId')
        description = self.parameters.get('Description')

        self.kms_backend.update_key_description(key_id, description)
        return json.dumps(None)

    def tag_resource(self):
        key_id = self.parameters.get('KeyId')
        tags = self.parameters.get('Tags')
        self.kms_backend.tag_resource(key_id, tags)
        return json.dumps({})

    def list_resource_tags(self):
        key_id = self.parameters.get('KeyId')
        tags = self.kms_backend.list_resource_tags(key_id)
        return json.dumps({
            "Tags": tags,
            "NextMarker": None,
            "Truncated": False,
        })

    def describe_key(self):
        key_id = self.parameters.get('KeyId')
        try:
            key = self.kms_backend.describe_key(
                self.kms_backend.get_key_id(key_id))
        except KeyError:
            headers = dict(self.headers)
            headers['status'] = 404
            return "{}", headers
        return json.dumps(key.to_dict())

    def list_keys(self):
        keys = self.kms_backend.list_keys()

        return json.dumps({
            "Keys": [
                {
                    "KeyArn": key.arn,
                    "KeyId": key.id,
                } for key in keys
            ],
            "NextMarker": None,
            "Truncated": False,
        })

    def create_alias(self):
        alias_name = self.parameters['AliasName']
        target_key_id = self.parameters['TargetKeyId']

        if not alias_name.startswith('alias/'):
            raise ValidationException('Invalid identifier')

        if alias_name in reserved_aliases:
            raise NotAuthorizedException()

        if ':' in alias_name:
            raise ValidationException('{alias_name} contains invalid characters for an alias'.format(alias_name=alias_name))

        if not re.match(r'^[a-zA-Z0-9:/_-]+$', alias_name):
            raise ValidationException("1 validation error detected: Value '{alias_name}' at 'aliasName' "
                                  "failed to satisfy constraint: Member must satisfy regular "
                                  "expression pattern: ^[a-zA-Z0-9:/_-]+$"
                                      .format(alias_name=alias_name))

        if self.kms_backend.alias_exists(target_key_id):
            raise ValidationException('Aliases must refer to keys. Not aliases')

        if self.kms_backend.alias_exists(alias_name):
            raise AlreadyExistsException('An alias with the name arn:aws:kms:{region}:012345678912:{alias_name} '
                                         'already exists'.format(region=self.region, alias_name=alias_name))

        self.kms_backend.add_alias(target_key_id, alias_name)

        return json.dumps(None)

    def delete_alias(self):
        alias_name = self.parameters['AliasName']

        if not alias_name.startswith('alias/'):
            raise ValidationException('Invalid identifier')

        if not self.kms_backend.alias_exists(alias_name):
            raise NotFoundException('Alias arn:aws:kms:{region}:012345678912:'
                                    '{alias_name} is not found.'.format(region=self.region, alias_name=alias_name))

        self.kms_backend.delete_alias(alias_name)

        return json.dumps(None)

    def list_aliases(self):
        region = self.region

        response_aliases = [
            {
                'AliasArn': u'arn:aws:kms:{region}:012345678912:{reserved_alias}'.format(region=region,
                                                                                         reserved_alias=reserved_alias),
                'AliasName': reserved_alias
            } for reserved_alias in reserved_aliases
        ]

        backend_aliases = self.kms_backend.get_all_aliases()
        for target_key_id, aliases in backend_aliases.items():
            for alias_name in aliases:
                response_aliases.append({
                    'AliasArn': u'arn:aws:kms:{region}:012345678912:{alias_name}'.format(region=region,
                                                                                         alias_name=alias_name),
                    'AliasName': alias_name,
                    'TargetKeyId': target_key_id,
                })

        return json.dumps({
            'Truncated': False,
            'Aliases': response_aliases,
        })

    def enable_key_rotation(self):
        key_id = self.parameters.get('KeyId')
        _assert_valid_key_id(self.kms_backend.get_key_id(key_id))
        try:
            self.kms_backend.enable_key_rotation(key_id)
        except KeyError:
            raise NotFoundException("Key 'arn:aws:kms:{region}:012345678912:key/"
                                    "{key_id}' does not exist".format(region=self.region, key_id=key_id))

        return json.dumps(None)

    def disable_key_rotation(self):
        key_id = self.parameters.get('KeyId')
        _assert_valid_key_id(self.kms_backend.get_key_id(key_id))
        try:
            self.kms_backend.disable_key_rotation(key_id)
        except KeyError:
            raise NotFoundException("Key 'arn:aws:kms:{region}:012345678912:key/"
                                    "{key_id}' does not exist".format(region=self.region, key_id=key_id))
        return json.dumps(None)

    def get_key_rotation_status(self):
        key_id = self.parameters.get('KeyId')
        _assert_valid_key_id(self.kms_backend.get_key_id(key_id))
        try:
            rotation_enabled = self.kms_backend.get_key_rotation_status(key_id)
        except KeyError:
            raise NotFoundException("Key 'arn:aws:kms:{region}:012345678912:key/"
                                    "{key_id}' does not exist".format(region=self.region, key_id=key_id))
        return json.dumps({'KeyRotationEnabled': rotation_enabled})

    def put_key_policy(self):
        key_id = self.parameters.get('KeyId')
        policy_name = self.parameters.get('PolicyName')
        policy = self.parameters.get('Policy')
        _assert_valid_key_id(self.kms_backend.get_key_id(key_id))
        _assert_default_policy(policy_name)

        try:
            self.kms_backend.put_key_policy(key_id, policy)
        except KeyError:
            raise NotFoundException("Key 'arn:aws:kms:{region}:012345678912:key/"
                                    "{key_id}' does not exist".format(region=self.region, key_id=key_id))

        return json.dumps(None)

    def get_key_policy(self):
        key_id = self.parameters.get('KeyId')
        policy_name = self.parameters.get('PolicyName')
        _assert_valid_key_id(self.kms_backend.get_key_id(key_id))
        _assert_default_policy(policy_name)

        try:
            return json.dumps({'Policy': self.kms_backend.get_key_policy(key_id)})
        except KeyError:
            raise NotFoundException("Key 'arn:aws:kms:{region}:012345678912:key/"
                                    "{key_id}' does not exist".format(region=self.region, key_id=key_id))

    def list_key_policies(self):
        key_id = self.parameters.get('KeyId')
        _assert_valid_key_id(self.kms_backend.get_key_id(key_id))
        try:
            self.kms_backend.describe_key(key_id)
        except KeyError:
            raise NotFoundException("Key 'arn:aws:kms:{region}:012345678912:key/"
                                    "{key_id}' does not exist".format(region=self.region, key_id=key_id))

        return json.dumps({'Truncated': False, 'PolicyNames': ['default']})

    def encrypt(self):
        """
        We perform no encryption, we just encode the value as base64 and then
        decode it in decrypt().
        """
        value = self.parameters.get("Plaintext")
        if isinstance(value, six.text_type):
            value = value.encode('utf-8')
        return json.dumps({"CiphertextBlob": base64.b64encode(value).decode("utf-8"), 'KeyId': 'key_id'})

    def decrypt(self):
        # TODO refuse decode if EncryptionContext is not the same as when it was encrypted / generated

        value = self.parameters.get("CiphertextBlob")
        try:
            return json.dumps({"Plaintext": base64.b64decode(value).decode("utf-8")})
        except UnicodeDecodeError:
            # Generate data key will produce random bytes which when decrypted is still returned as base64
            return json.dumps({"Plaintext": value})

    def disable_key(self):
        key_id = self.parameters.get('KeyId')
        _assert_valid_key_id(self.kms_backend.get_key_id(key_id))
        try:
            self.kms_backend.disable_key(key_id)
        except KeyError:
            raise NotFoundException("Key 'arn:aws:kms:{region}:012345678912:key/"
                                    "{key_id}' does not exist".format(region=self.region, key_id=key_id))
        return json.dumps(None)

    def enable_key(self):
        key_id = self.parameters.get('KeyId')
        _assert_valid_key_id(self.kms_backend.get_key_id(key_id))
        try:
            self.kms_backend.enable_key(key_id)
        except KeyError:
            raise NotFoundException("Key 'arn:aws:kms:{region}:012345678912:key/"
                                    "{key_id}' does not exist".format(region=self.region, key_id=key_id))
        return json.dumps(None)

    def cancel_key_deletion(self):
        key_id = self.parameters.get('KeyId')
        _assert_valid_key_id(self.kms_backend.get_key_id(key_id))
        try:
            self.kms_backend.cancel_key_deletion(key_id)
        except KeyError:
            raise NotFoundException("Key 'arn:aws:kms:{region}:012345678912:key/"
                                    "{key_id}' does not exist".format(region=self.region, key_id=key_id))
        return json.dumps({'KeyId': key_id})

    def schedule_key_deletion(self):
        key_id = self.parameters.get('KeyId')
        if self.parameters.get('PendingWindowInDays') is None:
            pending_window_in_days = 30
        else:
            pending_window_in_days = self.parameters.get('PendingWindowInDays')
        _assert_valid_key_id(self.kms_backend.get_key_id(key_id))
        try:
            return json.dumps({
                'KeyId': key_id,
                'DeletionDate': self.kms_backend.schedule_key_deletion(key_id, pending_window_in_days)
            })
        except KeyError:
            raise NotFoundException("Key 'arn:aws:kms:{region}:012345678912:key/"
                                    "{key_id}' does not exist".format(region=self.region, key_id=key_id))

    def generate_data_key(self):
        key_id = self.parameters.get('KeyId')
        encryption_context = self.parameters.get('EncryptionContext')
        number_of_bytes = self.parameters.get('NumberOfBytes')
        key_spec = self.parameters.get('KeySpec')
        grant_tokens = self.parameters.get('GrantTokens')

        # Param validation
        if key_id.startswith('alias'):
            if self.kms_backend.get_key_id_from_alias(key_id) is None:
                raise NotFoundException('Alias arn:aws:kms:{region}:012345678912:{alias_name} is not found.'.format(
                                        region=self.region, alias_name=key_id))
        else:
            if self.kms_backend.get_key_id(key_id) not in self.kms_backend.keys:
                raise NotFoundException('Invalid keyId')

        if number_of_bytes and (number_of_bytes > 1024 or number_of_bytes < 0):
            raise ValidationException("1 validation error detected: Value '2048' at 'numberOfBytes' failed "
                                  "to satisfy constraint: Member must have value less than or "
                                  "equal to 1024")

        if key_spec and key_spec not in ('AES_256', 'AES_128'):
            raise ValidationException("1 validation error detected: Value 'AES_257' at 'keySpec' failed "
                                  "to satisfy constraint: Member must satisfy enum value set: "
                                  "[AES_256, AES_128]")
        if not key_spec and not number_of_bytes:
            raise ValidationException("Please specify either number of bytes or key spec.")
        if key_spec and number_of_bytes:
            raise ValidationException("Please specify either number of bytes or key spec.")

        plaintext, key_arn = self.kms_backend.generate_data_key(key_id, encryption_context,
                                                                number_of_bytes, key_spec, grant_tokens)

        plaintext = base64.b64encode(plaintext).decode()

        return json.dumps({
            'CiphertextBlob': plaintext,
            'Plaintext': plaintext,
            'KeyId': key_arn  # not alias
        })

    def generate_data_key_without_plaintext(self):
        result = json.loads(self.generate_data_key())
        del result['Plaintext']

        return json.dumps(result)


def _assert_valid_key_id(key_id):
    if not re.match(r'^[A-F0-9]{8}-[A-F0-9]{4}-[A-F0-9]{4}-[A-F0-9]{4}-[A-F0-9]{12}$', key_id, re.IGNORECASE):
        raise NotFoundException('Invalid keyId')


def _assert_default_policy(policy_name):
    if policy_name != 'default':
        raise NotFoundException("No such policy exists")



from __future__ import unicode_literals
from .responses import CognitoIdpResponse, CognitoIdpJsonWebKeyResponse

url_bases = [
    "https?://cognito-idp.(.+).amazonaws.com",
]

url_paths = {
    '{0}/$': CognitoIdpResponse.dispatch,
    '{0}/<user_pool_id>/.well-known/jwks.json$': CognitoIdpJsonWebKeyResponse().serve_json_web_key,
}



from __future__ import unicode_literals
import boto3
import sure  # noqa
import re

from moto import mock_opsworks


@mock_opsworks
def test_create_stack_response():
    client = boto3.client('opsworks', region_name='us-east-1')
    response = client.create_stack(
        Name="test_stack_1",
        Region="us-east-1",
        ServiceRoleArn="service_arn",
        DefaultInstanceProfileArn="profile_arn"
    )
    response.should.contain("StackId")


@mock_opsworks
def test_describe_stacks():
    client = boto3.client('opsworks', region_name='us-east-1')
    for i in range(1, 4):
        client.create_stack(
            Name="test_stack_{0}".format(i),
            Region="us-east-1",
            ServiceRoleArn="service_arn",
            DefaultInstanceProfileArn="profile_arn"
        )

    response = client.describe_stacks()
    response['Stacks'].should.have.length_of(3)
    for stack in response['Stacks']:
        stack['ServiceRoleArn'].should.equal("service_arn")
        stack['DefaultInstanceProfileArn'].should.equal("profile_arn")

    _id = response['Stacks'][0]['StackId']
    response = client.describe_stacks(StackIds=[_id])
    response['Stacks'].should.have.length_of(1)
    response['Stacks'][0]['Arn'].should.contain(_id)

    # ClientError/ResourceNotFoundException
    client.describe_stacks.when.called_with(StackIds=["foo"]).should.throw(
        Exception, re.compile(r'foo')
    )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from __future__ import absolute_import

import functools
import inspect
import os
import re
import six
from io import BytesIO
from collections import defaultdict
from botocore.handlers import BUILTIN_HANDLERS
from botocore.awsrequest import AWSResponse

from moto import settings
import responses
from moto.packages.httpretty import HTTPretty
from .utils import (
    convert_httpretty_response,
    convert_regex_to_flask_path,
    convert_flask_to_responses_response,
)


# "Mock" the AWS credentials as they can't be mocked in Botocore currently
os.environ.setdefault("AWS_ACCESS_KEY_ID", "foobar_key")
os.environ.setdefault("AWS_SECRET_ACCESS_KEY", "foobar_secret")


class BaseMockAWS(object):
    nested_count = 0

    def __init__(self, backends):
        self.backends = backends

        self.backends_for_urls = {}
        from moto.backends import BACKENDS
        default_backends = {
            "instance_metadata": BACKENDS['instance_metadata']['global'],
            "moto_api": BACKENDS['moto_api']['global'],
        }
        self.backends_for_urls.update(self.backends)
        self.backends_for_urls.update(default_backends)

        if self.__class__.nested_count == 0:
            self.reset()

    def __call__(self, func, reset=True):
        if inspect.isclass(func):
            return self.decorate_class(func)
        return self.decorate_callable(func, reset)

    def __enter__(self):
        self.start()

    def __exit__(self, *args):
        self.stop()

    def start(self, reset=True):
        self.__class__.nested_count += 1
        if reset:
            for backend in self.backends.values():
                backend.reset()

        self.enable_patching()

    def stop(self):
        self.__class__.nested_count -= 1

        if self.__class__.nested_count < 0:
            raise RuntimeError('Called stop() before start().')

        if self.__class__.nested_count == 0:
            self.disable_patching()

    def decorate_callable(self, func, reset):
        def wrapper(*args, **kwargs):
            self.start(reset=reset)
            try:
                result = func(*args, **kwargs)
            finally:
                self.stop()
            return result
        functools.update_wrapper(wrapper, func)
        wrapper.__wrapped__ = func
        return wrapper

    def decorate_class(self, klass):
        for attr in dir(klass):
            if attr.startswith("_"):
                continue

            attr_value = getattr(klass, attr)
            if not hasattr(attr_value, "__call__"):
                continue

            # Check if this is a classmethod. If so, skip patching
            if inspect.ismethod(attr_value) and attr_value.__self__ is klass:
                continue

            # Check if this is a staticmethod. If so, skip patching
            for cls in inspect.getmro(klass):
                if attr_value.__name__ not in cls.__dict__:
                    continue
                bound_attr_value = cls.__dict__[attr_value.__name__]
                if not isinstance(bound_attr_value, staticmethod):
                    break
            else:
                # It is a staticmethod, skip patching
                continue

            try:
                setattr(klass, attr, self(attr_value, reset=False))
            except TypeError:
                # Sometimes we can't set this for built-in types
                continue
        return klass


class HttprettyMockAWS(BaseMockAWS):

    def reset(self):
        HTTPretty.reset()

    def enable_patching(self):
        if not HTTPretty.is_enabled():
            HTTPretty.enable()

        for method in HTTPretty.METHODS:
            for backend in self.backends_for_urls.values():
                for key, value in backend.urls.items():
                    HTTPretty.register_uri(
                        method=method,
                        uri=re.compile(key),
                        body=convert_httpretty_response(value),
                    )

    def disable_patching(self):
        HTTPretty.disable()
        HTTPretty.reset()


RESPONSES_METHODS = [responses.GET, responses.DELETE, responses.HEAD,
                     responses.OPTIONS, responses.PATCH, responses.POST, responses.PUT]


class CallbackResponse(responses.CallbackResponse):
    '''
    Need to subclass so we can change a couple things
    '''
    def get_response(self, request):
        '''
        Need to override this so we can pass decode_content=False
        '''
        headers = self.get_headers()

        result = self.callback(request)
        if isinstance(result, Exception):
            raise result

        status, r_headers, body = result
        body = responses._handle_body(body)
        headers.update(r_headers)

        return responses.HTTPResponse(
            status=status,
            reason=six.moves.http_client.responses.get(status),
            body=body,
            headers=headers,
            preload_content=False,
            # Need to not decode_content to mimic requests
            decode_content=False,
        )

    def _url_matches(self, url, other, match_querystring=False):
        '''
        Need to override this so we can fix querystrings breaking regex matching
        '''
        if not match_querystring:
            other = other.split('?', 1)[0]

        if responses._is_string(url):
            if responses._has_unicode(url):
                url = responses._clean_unicode(url)
                if not isinstance(other, six.text_type):
                    other = other.encode('ascii').decode('utf8')
            return self._url_matches_strict(url, other)
        elif isinstance(url, responses.Pattern) and url.match(other):
            return True
        else:
            return False


botocore_mock = responses.RequestsMock(assert_all_requests_are_fired=False, target='botocore.vendored.requests.adapters.HTTPAdapter.send')
responses_mock = responses._default_mock


class ResponsesMockAWS(BaseMockAWS):
    def reset(self):
        botocore_mock.reset()
        responses_mock.reset()

    def enable_patching(self):
        if not hasattr(botocore_mock, '_patcher') or not hasattr(botocore_mock._patcher, 'target'):
            # Check for unactivated patcher
            botocore_mock.start()

        if not hasattr(responses_mock, '_patcher') or not hasattr(responses_mock._patcher, 'target'):
            responses_mock.start()

        for method in RESPONSES_METHODS:
            for backend in self.backends_for_urls.values():
                for key, value in backend.urls.items():
                    responses_mock.add(
                        CallbackResponse(
                            method=method,
                            url=re.compile(key),
                            callback=convert_flask_to_responses_response(value),
                            stream=True,
                            match_querystring=False,
                        )
                    )
                    botocore_mock.add(
                        CallbackResponse(
                            method=method,
                            url=re.compile(key),
                            callback=convert_flask_to_responses_response(value),
                            stream=True,
                            match_querystring=False,
                        )
                    )

    def disable_patching(self):
        try:
            botocore_mock.stop()
        except RuntimeError:
            pass

        try:
            responses_mock.stop()
        except RuntimeError:
            pass


BOTOCORE_HTTP_METHODS = [
    'GET', 'DELETE', 'HEAD', 'OPTIONS', 'PATCH', 'POST', 'PUT'
]


class MockRawResponse(BytesIO):
    def __init__(self, input):
        if isinstance(input, six.text_type):
            input = input.encode('utf-8')
        super(MockRawResponse, self).__init__(input)

    def stream(self, **kwargs):
        contents = self.read()
        while contents:
            yield contents
            contents = self.read()


class BotocoreStubber(object):
    def __init__(self):
        self.enabled = False
        self.methods = defaultdict(list)

    def reset(self):
        self.methods.clear()

    def register_response(self, method, pattern, response):
        matchers = self.methods[method]
        matchers.append((pattern, response))

    def __call__(self, event_name, request, **kwargs):
        if not self.enabled:
            return None

        response = None
        response_callback = None
        found_index = None
        matchers = self.methods.get(request.method)

        base_url = request.url.split('?', 1)[0]
        for i, (pattern, callback) in enumerate(matchers):
            if pattern.match(base_url):
                if found_index is None:
                    found_index = i
                    response_callback = callback
                else:
                    matchers.pop(found_index)
                    break

        if response_callback is not None:
            for header, value in request.headers.items():
                if isinstance(value, six.binary_type):
                    request.headers[header] = value.decode('utf-8')
            status, headers, body = response_callback(request, request.url, request.headers)
            body = MockRawResponse(body)
            response = AWSResponse(request.url, status, headers, body)

        return response


botocore_stubber = BotocoreStubber()
BUILTIN_HANDLERS.append(('before-send', botocore_stubber))


class BotocoreEventMockAWS(BaseMockAWS):
    def reset(self):
        botocore_stubber.reset()
        responses_mock.reset()

    def enable_patching(self):
        botocore_stubber.enabled = True
        for method in BOTOCORE_HTTP_METHODS:
            for backend in self.backends_for_urls.values():
                for key, value in backend.urls.items():
                    pattern = re.compile(key)
                    botocore_stubber.register_response(method, pattern, value)

        if not hasattr(responses_mock, '_patcher') or not hasattr(responses_mock._patcher, 'target'):
            responses_mock.start()

        for method in RESPONSES_METHODS:
            # for backend in default_backends.values():
            for backend in self.backends_for_urls.values():
                for key, value in backend.urls.items():
                    responses_mock.add(
                        CallbackResponse(
                            method=method,
                            url=re.compile(key),
                            callback=convert_flask_to_responses_response(value),
                            stream=True,
                            match_querystring=False,
                        )
                    )

    def disable_patching(self):
        botocore_stubber.enabled = False
        self.reset()

        try:
            responses_mock.stop()
        except RuntimeError:
            pass


MockAWS = BotocoreEventMockAWS


class ServerModeMockAWS(BaseMockAWS):

    def reset(self):
        import requests
        requests.post("http://localhost:5000/moto-api/reset")

    def enable_patching(self):
        if self.__class__.nested_count == 1:
            # Just started
            self.reset()

        from boto3 import client as real_boto3_client, resource as real_boto3_resource
        import mock

        def fake_boto3_client(*args, **kwargs):
            if 'endpoint_url' not in kwargs:
                kwargs['endpoint_url'] = "http://localhost:5000"
            return real_boto3_client(*args, **kwargs)

        def fake_boto3_resource(*args, **kwargs):
            if 'endpoint_url' not in kwargs:
                kwargs['endpoint_url'] = "http://localhost:5000"
            return real_boto3_resource(*args, **kwargs)

        def fake_httplib_send_output(self, message_body=None, *args, **kwargs):
            def _convert_to_bytes(mixed_buffer):
                bytes_buffer = []
                for chunk in mixed_buffer:
                    if isinstance(chunk, six.text_type):
                        bytes_buffer.append(chunk.encode('utf-8'))
                    else:
                        bytes_buffer.append(chunk)
                msg = b"\r\n".join(bytes_buffer)
                return msg

            self._buffer.extend((b"", b""))
            msg = _convert_to_bytes(self._buffer)
            del self._buffer[:]
            if isinstance(message_body, bytes):
                msg += message_body
                message_body = None
            self.send(msg)
            # if self._expect_header_set:
            #     read, write, exc = select.select([self.sock], [], [self.sock], 1)
            #     if read:
            #         self._handle_expect_response(message_body)
            #         return
            if message_body is not None:
                self.send(message_body)

        self._client_patcher = mock.patch('boto3.client', fake_boto3_client)
        self._resource_patcher = mock.patch('boto3.resource', fake_boto3_resource)
        if six.PY2:
            self._httplib_patcher = mock.patch('httplib.HTTPConnection._send_output', fake_httplib_send_output)

        self._client_patcher.start()
        self._resource_patcher.start()
        if six.PY2:
            self._httplib_patcher.start()

    def disable_patching(self):
        if self._client_patcher:
            self._client_patcher.stop()
            self._resource_patcher.stop()
            if six.PY2:
                self._httplib_patcher.stop()


class Model(type):

    def __new__(self, clsname, bases, namespace):
        cls = super(Model, self).__new__(self, clsname, bases, namespace)
        cls.__models__ = {}
        for name, value in namespace.items():
            model = getattr(value, "__returns_model__", False)
            if model is not False:
                cls.__models__[model] = name
        for base in bases:
            cls.__models__.update(getattr(base, "__models__", {}))
        return cls

    @staticmethod
    def prop(model_name):
        """ decorator to mark a class method as returning model values """
        def dec(f):
            f.__returns_model__ = model_name
            return f
        return dec


model_data = defaultdict(dict)


class InstanceTrackerMeta(type):
    def __new__(meta, name, bases, dct):
        cls = super(InstanceTrackerMeta, meta).__new__(meta, name, bases, dct)
        if name == 'BaseModel':
            return cls

        service = cls.__module__.split(".")[1]
        if name not in model_data[service]:
            model_data[service][name] = cls
        cls.instances = []
        return cls


@six.add_metaclass(InstanceTrackerMeta)
class BaseModel(object):
    def __new__(cls, *args, **kwargs):
        instance = super(BaseModel, cls).__new__(cls)
        cls.instances.append(instance)
        return instance


class BaseBackend(object):

    def reset(self):
        for service, models in model_data.items():
            for model_name, model in models.items():
                model.instances = []
        self.__dict__ = {}
        self.__init__()

    @property
    def _url_module(self):
        backend_module = self.__class__.__module__
        backend_urls_module_name = backend_module.replace("models", "urls")
        backend_urls_module = __import__(backend_urls_module_name, fromlist=[
                                         'url_bases', 'url_paths'])
        return backend_urls_module

    @property
    def urls(self):
        """
        A dictionary of the urls to be mocked with this service and the handlers
        that should be called in their place
        """
        url_bases = self._url_module.url_bases
        unformatted_paths = self._url_module.url_paths

        urls = {}
        for url_base in url_bases:
            for url_path, handler in unformatted_paths.items():
                url = url_path.format(url_base)
                urls[url] = handler

        return urls

    @property
    def url_paths(self):
        """
        A dictionary of the paths of the urls to be mocked with this service and
        the handlers that should be called in their place
        """
        unformatted_paths = self._url_module.url_paths

        paths = {}
        for unformatted_path, handler in unformatted_paths.items():
            path = unformatted_path.format("")
            paths[path] = handler

        return paths

    @property
    def url_bases(self):
        """
        A list containing the url_bases extracted from urls.py
        """
        return self._url_module.url_bases

    @property
    def flask_paths(self):
        """
        The url paths that will be used for the flask server
        """
        paths = {}
        for url_path, handler in self.url_paths.items():
            url_path = convert_regex_to_flask_path(url_path)
            paths[url_path] = handler

        return paths

    def decorator(self, func=None):
        if settings.TEST_SERVER_MODE:
            mocked_backend = ServerModeMockAWS({'global': self})
        else:
            mocked_backend = MockAWS({'global': self})

        if func:
            return mocked_backend(func)
        else:
            return mocked_backend

    def deprecated_decorator(self, func=None):
        if func:
            return HttprettyMockAWS({'global': self})(func)
        else:
            return HttprettyMockAWS({'global': self})


class base_decorator(object):
    mock_backend = MockAWS

    def __init__(self, backends):
        self.backends = backends

    def __call__(self, func=None):
        if self.mock_backend != HttprettyMockAWS and settings.TEST_SERVER_MODE:
            mocked_backend = ServerModeMockAWS(self.backends)
        else:
            mocked_backend = self.mock_backend(self.backends)

        if func:
            return mocked_backend(func)
        else:
            return mocked_backend


class deprecated_base_decorator(base_decorator):
    mock_backend = HttprettyMockAWS


class MotoAPIBackend(BaseBackend):

    def reset(self):
        from moto.backends import BACKENDS
        for name, backends in BACKENDS.items():
            if name == "moto_api":
                continue
            for region_name, backend in backends.items():
                backend.reset()
        self.__init__()


moto_api_backend = MotoAPIBackend()



from __future__ import unicode_literals
from .responses import SimpleSystemManagerResponse

url_bases = [
    "https?://ssm.(.+).amazonaws.com",
    "https?://ssm.(.+).amazonaws.com.cn",
]

url_paths = {
    '{0}/$': SimpleSystemManagerResponse.dispatch,
}



from __future__ import unicode_literals
from .models import ecs_backends
from ..core.models import base_decorator, deprecated_base_decorator

ecs_backend = ecs_backends['us-east-1']
mock_ecs = base_decorator(ecs_backends)
mock_ecs_deprecated = deprecated_base_decorator(ecs_backends)



import json
from moto.core.utils import amzn_request_id
from moto.core.responses import BaseResponse
from .models import cloudwatch_backends
from dateutil.parser import parse as dtparse


class CloudWatchResponse(BaseResponse):

    @property
    def cloudwatch_backend(self):
        return cloudwatch_backends[self.region]

    def _error(self, code, message, status=400):
        template = self.response_template(ERROR_RESPONSE_TEMPLATE)
        return template.render(code=code, message=message), dict(status=status)

    @amzn_request_id
    def put_metric_alarm(self):
        name = self._get_param('AlarmName')
        namespace = self._get_param('Namespace')
        metric_name = self._get_param('MetricName')
        comparison_operator = self._get_param('ComparisonOperator')
        evaluation_periods = self._get_param('EvaluationPeriods')
        period = self._get_param('Period')
        threshold = self._get_param('Threshold')
        statistic = self._get_param('Statistic')
        description = self._get_param('AlarmDescription')
        dimensions = self._get_list_prefix('Dimensions.member')
        alarm_actions = self._get_multi_param('AlarmActions.member')
        ok_actions = self._get_multi_param('OKActions.member')
        insufficient_data_actions = self._get_multi_param(
            "InsufficientDataActions.member")
        unit = self._get_param('Unit')
        alarm = self.cloudwatch_backend.put_metric_alarm(name, namespace, metric_name,
                                                         comparison_operator,
                                                         evaluation_periods, period,
                                                         threshold, statistic,
                                                         description, dimensions,
                                                         alarm_actions, ok_actions,
                                                         insufficient_data_actions,
                                                         unit)
        template = self.response_template(PUT_METRIC_ALARM_TEMPLATE)
        return template.render(alarm=alarm)

    @amzn_request_id
    def describe_alarms(self):
        action_prefix = self._get_param('ActionPrefix')
        alarm_name_prefix = self._get_param('AlarmNamePrefix')
        alarm_names = self._get_multi_param('AlarmNames.member')
        state_value = self._get_param('StateValue')

        if action_prefix:
            alarms = self.cloudwatch_backend.get_alarms_by_action_prefix(
                action_prefix)
        elif alarm_name_prefix:
            alarms = self.cloudwatch_backend.get_alarms_by_alarm_name_prefix(
                alarm_name_prefix)
        elif alarm_names:
            alarms = self.cloudwatch_backend.get_alarms_by_alarm_names(alarm_names)
        elif state_value:
            alarms = self.cloudwatch_backend.get_alarms_by_state_value(state_value)
        else:
            alarms = self.cloudwatch_backend.get_all_alarms()

        template = self.response_template(DESCRIBE_ALARMS_TEMPLATE)
        return template.render(alarms=alarms)

    @amzn_request_id
    def delete_alarms(self):
        alarm_names = self._get_multi_param('AlarmNames.member')
        self.cloudwatch_backend.delete_alarms(alarm_names)
        template = self.response_template(DELETE_METRIC_ALARMS_TEMPLATE)
        return template.render()

    @amzn_request_id
    def put_metric_data(self):
        namespace = self._get_param('Namespace')
        metric_data = self._get_multi_param('MetricData.member')

        self.cloudwatch_backend.put_metric_data(namespace, metric_data)
        template = self.response_template(PUT_METRIC_DATA_TEMPLATE)
        return template.render()

    @amzn_request_id
    def get_metric_statistics(self):
        namespace = self._get_param('Namespace')
        metric_name = self._get_param('MetricName')
        start_time = dtparse(self._get_param('StartTime'))
        end_time = dtparse(self._get_param('EndTime'))
        period = int(self._get_param('Period'))
        statistics = self._get_multi_param("Statistics.member")

        # Unsupported Parameters (To Be Implemented)
        unit = self._get_param('Unit')
        extended_statistics = self._get_param('ExtendedStatistics')
        dimensions = self._get_param('Dimensions')
        if unit or extended_statistics or dimensions:
            raise NotImplemented()

        # TODO: this should instead throw InvalidParameterCombination
        if not statistics:
            raise NotImplemented("Must specify either Statistics or ExtendedStatistics")

        datapoints = self.cloudwatch_backend.get_metric_statistics(namespace, metric_name, start_time, end_time, period, statistics)
        template = self.response_template(GET_METRIC_STATISTICS_TEMPLATE)
        return template.render(label=metric_name, datapoints=datapoints)

    @amzn_request_id
    def list_metrics(self):
        metrics = self.cloudwatch_backend.get_all_metrics()
        template = self.response_template(LIST_METRICS_TEMPLATE)
        return template.render(metrics=metrics)

    @amzn_request_id
    def delete_dashboards(self):
        dashboards = self._get_multi_param('DashboardNames.member')
        if dashboards is None:
            return self._error('InvalidParameterValue', 'Need at least 1 dashboard')

        status, error = self.cloudwatch_backend.delete_dashboards(dashboards)
        if not status:
            return self._error('ResourceNotFound', error)

        template = self.response_template(DELETE_DASHBOARD_TEMPLATE)
        return template.render()

    @amzn_request_id
    def describe_alarm_history(self):
        raise NotImplementedError()

    @amzn_request_id
    def describe_alarms_for_metric(self):
        raise NotImplementedError()

    @amzn_request_id
    def disable_alarm_actions(self):
        raise NotImplementedError()

    @amzn_request_id
    def enable_alarm_actions(self):
        raise NotImplementedError()

    @amzn_request_id
    def get_dashboard(self):
        dashboard_name = self._get_param('DashboardName')

        dashboard = self.cloudwatch_backend.get_dashboard(dashboard_name)
        if dashboard is None:
            return self._error('ResourceNotFound', 'Dashboard does not exist')

        template = self.response_template(GET_DASHBOARD_TEMPLATE)
        return template.render(dashboard=dashboard)

    @amzn_request_id
    def list_dashboards(self):
        prefix = self._get_param('DashboardNamePrefix', '')

        dashboards = self.cloudwatch_backend.list_dashboards(prefix)

        template = self.response_template(LIST_DASHBOARD_RESPONSE)
        return template.render(dashboards=dashboards)

    @amzn_request_id
    def put_dashboard(self):
        name = self._get_param('DashboardName')
        body = self._get_param('DashboardBody')

        try:
            json.loads(body)
        except ValueError:
            return self._error('InvalidParameterInput', 'Body is invalid JSON')

        self.cloudwatch_backend.put_dashboard(name, body)

        template = self.response_template(PUT_DASHBOARD_RESPONSE)
        return template.render()

    @amzn_request_id
    def set_alarm_state(self):
        alarm_name = self._get_param('AlarmName')
        reason = self._get_param('StateReason')
        reason_data = self._get_param('StateReasonData')
        state_value = self._get_param('StateValue')

        self.cloudwatch_backend.set_alarm_state(alarm_name, reason, reason_data, state_value)

        template = self.response_template(SET_ALARM_STATE_TEMPLATE)
        return template.render()


PUT_METRIC_ALARM_TEMPLATE = """<PutMetricAlarmResponse xmlns="http://monitoring.amazonaws.com/doc/2010-08-01/">
   <ResponseMetadata>
      <RequestId>
         {{ request_id }}
      </RequestId>
   </ResponseMetadata>
</PutMetricAlarmResponse>"""

DESCRIBE_ALARMS_TEMPLATE = """<DescribeAlarmsResponse xmlns="http://monitoring.amazonaws.com/doc/2010-08-01/">
    <DescribeAlarmsResult>
        <MetricAlarms>
            {% for alarm in alarms %}
            <member>
                <ActionsEnabled>{{ alarm.actions_enabled }}</ActionsEnabled>
                <AlarmActions>
                    {% for action in alarm.alarm_actions %}
                    <member>{{ action }}</member>
                    {% endfor %}
                </AlarmActions>
                <AlarmArn>{{ alarm.arn }}</AlarmArn>
                <AlarmConfigurationUpdatedTimestamp>{{ alarm.configuration_updated_timestamp }}</AlarmConfigurationUpdatedTimestamp>
                <AlarmDescription>{{ alarm.description }}</AlarmDescription>
                <AlarmName>{{ alarm.name }}</AlarmName>
                <ComparisonOperator>{{ alarm.comparison_operator }}</ComparisonOperator>
                <Dimensions>
                    {% for dimension in alarm.dimensions %}
                    <member>
                        <Name>{{ dimension.name }}</Name>
                        <Value>{{ dimension.value }}</Value>
                    </member>
                    {% endfor %}
                </Dimensions>
                <EvaluationPeriods>{{ alarm.evaluation_periods }}</EvaluationPeriods>
                <InsufficientDataActions>
                    {% for action in alarm.insufficient_data_actions %}
                    <member>{{ action }}</member>
                    {% endfor %}
                </InsufficientDataActions>
                <MetricName>{{ alarm.metric_name }}</MetricName>
                <Namespace>{{ alarm.namespace }}</Namespace>
                <OKActions>
                    {% for action in alarm.ok_actions %}
                    <member>{{ action }}</member>
                    {% endfor %}
                </OKActions>
                <Period>{{ alarm.period }}</Period>
                <StateReason>{{ alarm.state_reason }}</StateReason>
                <StateReasonData>{{ alarm.state_reason_data }}</StateReasonData>
                <StateUpdatedTimestamp>{{ alarm.state_updated_timestamp }}</StateUpdatedTimestamp>
                <StateValue>{{ alarm.state_value }}</StateValue>
                <Statistic>{{ alarm.statistic }}</Statistic>
                <Threshold>{{ alarm.threshold }}</Threshold>
                <Unit>{{ alarm.unit }}</Unit>
            </member>
            {% endfor %}
        </MetricAlarms>
    </DescribeAlarmsResult>
</DescribeAlarmsResponse>"""

DELETE_METRIC_ALARMS_TEMPLATE = """<DeleteMetricAlarmResponse xmlns="http://monitoring.amazonaws.com/doc/2010-08-01/">
   <ResponseMetadata>
      <RequestId>
         {{ request_id }}
      </RequestId>
   </ResponseMetadata>
</DeleteMetricAlarmResponse>"""

PUT_METRIC_DATA_TEMPLATE = """<PutMetricDataResponse xmlns="http://monitoring.amazonaws.com/doc/2010-08-01/">
   <ResponseMetadata>
      <RequestId>
         {{ request_id }}
      </RequestId>
   </ResponseMetadata>
</PutMetricDataResponse>"""

GET_METRIC_STATISTICS_TEMPLATE = """<GetMetricStatisticsResponse xmlns="http://monitoring.amazonaws.com/doc/2010-08-01/">
   <ResponseMetadata>
      <RequestId>
         {{ request_id }}
      </RequestId>
   </ResponseMetadata>

  <GetMetricStatisticsResult>
      <Label>{{ label }}</Label>
      <Datapoints>
        {% for datapoint in datapoints %}
            <member>
              {% if datapoint.sum is not none %}
              <Sum>{{ datapoint.sum }}</Sum>
              {% endif %}

              {% if datapoint.average is not none %}
              <Average>{{ datapoint.average }}</Average>
              {% endif %}

              {% if datapoint.maximum is not none %}
              <Maximum>{{ datapoint.maximum }}</Maximum>
              {% endif %}

              {% if datapoint.minimum is not none %}
              <Minimum>{{ datapoint.minimum }}</Minimum>
              {% endif %}

              {% if datapoint.sample_count is not none %}
              <SampleCount>{{ datapoint.sample_count }}</SampleCount>
              {% endif %}

              {% if datapoint.extended_statistics is not none %}
              <ExtendedStatistics>{{ datapoint.extended_statistics }}</ExtendedStatistics>
              {% endif %}

              <Timestamp>{{ datapoint.timestamp }}</Timestamp>
              <Unit>{{ datapoint.unit }}</Unit>
            </member>
        {% endfor %}
      </Datapoints>
    </GetMetricStatisticsResult>
</GetMetricStatisticsResponse>"""

LIST_METRICS_TEMPLATE = """<ListMetricsResponse xmlns="http://monitoring.amazonaws.com/doc/2010-08-01/">
    <ListMetricsResult>
        <Metrics>
            {% for metric in metrics %}
            <member>
                <Dimensions>
                    {% for dimension in metric.dimensions %}
                    <member>
                        <Name>{{ dimension.name }}</Name>
                        <Value>{{ dimension.value }}</Value>
                    </member>
                    {% endfor %}
                </Dimensions>
                <MetricName>{{ metric.name }}</MetricName>
                <Namespace>{{ metric.namespace }}</Namespace>
            </member>
            {% endfor %}
        </Metrics>
        <NextToken>
            96e88479-4662-450b-8a13-239ded6ce9fe
        </NextToken>
    </ListMetricsResult>
</ListMetricsResponse>"""

PUT_DASHBOARD_RESPONSE = """<PutDashboardResponse xmlns="http://monitoring.amazonaws.com/doc/2010-08-01/">
  <PutDashboardResult>
    <DashboardValidationMessages/>
  </PutDashboardResult>
  <ResponseMetadata>
    <RequestId>{{ request_id }}</RequestId>
  </ResponseMetadata>
</PutDashboardResponse>"""

LIST_DASHBOARD_RESPONSE = """<ListDashboardsResponse xmlns="http://monitoring.amazonaws.com/doc/2010-08-01/">
  <ListDashboardsResult>
    <DashboardEntries>
      {% for dashboard in dashboards %}
      <member>
        <DashboardArn>{{ dashboard.arn }}</DashboardArn>
        <LastModified>{{ dashboard.last_modified_iso }}</LastModified>
        <Size>{{ dashboard.size }}</Size>
        <DashboardName>{{ dashboard.name }}</DashboardName>
      </member>
      {% endfor %}
    </DashboardEntries>
  </ListDashboardsResult>
  <ResponseMetadata>
    <RequestId>{{ request_id }}</RequestId>
  </ResponseMetadata>
</ListDashboardsResponse>"""

DELETE_DASHBOARD_TEMPLATE = """<DeleteDashboardsResponse xmlns="http://monitoring.amazonaws.com/doc/2010-08-01/">
  <DeleteDashboardsResult/>
  <ResponseMetadata>
    <RequestId>{{ request_id }}</RequestId>
  </ResponseMetadata>
</DeleteDashboardsResponse>"""

GET_DASHBOARD_TEMPLATE = """<GetDashboardResponse xmlns="http://monitoring.amazonaws.com/doc/2010-08-01/">
  <GetDashboardResult>
    <DashboardArn>{{ dashboard.arn }}</DashboardArn>
    <DashboardBody>{{ dashboard.body }}</DashboardBody>
    <DashboardName>{{ dashboard.name }}</DashboardName>
  </GetDashboardResult>
  <ResponseMetadata>
    <RequestId>{{ request_id }}</RequestId>
  </ResponseMetadata>
</GetDashboardResponse>
"""

SET_ALARM_STATE_TEMPLATE = """<SetAlarmStateResponse xmlns="http://monitoring.amazonaws.com/doc/2010-08-01/">
  <ResponseMetadata>
    <RequestId>{{ request_id }}</RequestId>
  </ResponseMetadata>
</SetAlarmStateResponse>"""

ERROR_RESPONSE_TEMPLATE = """<ErrorResponse xmlns="http://monitoring.amazonaws.com/doc/2010-08-01/">
  <Error>
    <Type>Sender</Type>
    <Code>{{ code }}</Code>
    <Message>{{ message }}</Message>
  </Error>
  <RequestId>{{ request_id }}</RequestId>
</ErrorResponse>"""



from __future__ import unicode_literals

import boto3
import json
import six
import sure   # noqa
from botocore.exceptions import ClientError
from nose.tools import assert_raises

from moto import mock_organizations
from moto.organizations import utils
from .organizations_test_utils import (
    validate_organization,
    validate_roots,
    validate_organizational_unit,
    validate_account,
    validate_create_account_status,
    validate_service_control_policy,
    validate_policy_summary,
)


@mock_organizations
def test_create_organization():
    client = boto3.client('organizations', region_name='us-east-1')
    response = client.create_organization(FeatureSet='ALL')
    validate_organization(response)
    response['Organization']['FeatureSet'].should.equal('ALL')


@mock_organizations
def test_describe_organization():
    client = boto3.client('organizations', region_name='us-east-1')
    client.create_organization(FeatureSet='ALL')
    response = client.describe_organization()
    validate_organization(response)


@mock_organizations
def test_describe_organization_exception():
    client = boto3.client('organizations', region_name='us-east-1')
    with assert_raises(ClientError) as e:
        response = client.describe_organization()
    ex = e.exception
    ex.operation_name.should.equal('DescribeOrganization')
    ex.response['Error']['Code'].should.equal('400')
    ex.response['Error']['Message'].should.contain('AWSOrganizationsNotInUseException')


# Organizational Units

@mock_organizations
def test_list_roots():
    client = boto3.client('organizations', region_name='us-east-1')
    org = client.create_organization(FeatureSet='ALL')['Organization']
    response = client.list_roots()
    validate_roots(org, response)


@mock_organizations
def test_create_organizational_unit():
    client = boto3.client('organizations', region_name='us-east-1')
    org = client.create_organization(FeatureSet='ALL')['Organization']
    root_id = client.list_roots()['Roots'][0]['Id']
    ou_name = 'ou01'
    response = client.create_organizational_unit(
        ParentId=root_id,
        Name=ou_name,
    )
    validate_organizational_unit(org, response)
    response['OrganizationalUnit']['Name'].should.equal(ou_name)


@mock_organizations
def test_describe_organizational_unit():
    client = boto3.client('organizations', region_name='us-east-1')
    org = client.create_organization(FeatureSet='ALL')['Organization']
    root_id = client.list_roots()['Roots'][0]['Id']
    ou_id = client.create_organizational_unit(
        ParentId=root_id,
        Name='ou01',
    )['OrganizationalUnit']['Id']
    response = client.describe_organizational_unit(OrganizationalUnitId=ou_id)
    validate_organizational_unit(org, response)


@mock_organizations
def test_describe_organizational_unit_exception():
    client = boto3.client('organizations', region_name='us-east-1')
    org = client.create_organization(FeatureSet='ALL')['Organization']
    with assert_raises(ClientError) as e:
        response = client.describe_organizational_unit(
            OrganizationalUnitId=utils.make_random_root_id()
        )
    ex = e.exception
    ex.operation_name.should.equal('DescribeOrganizationalUnit')
    ex.response['Error']['Code'].should.equal('400')
    ex.response['Error']['Message'].should.contain('OrganizationalUnitNotFoundException')


@mock_organizations
def test_list_organizational_units_for_parent():
    client = boto3.client('organizations', region_name='us-east-1')
    org = client.create_organization(FeatureSet='ALL')['Organization']
    root_id = client.list_roots()['Roots'][0]['Id']
    client.create_organizational_unit(ParentId=root_id, Name='ou01')
    client.create_organizational_unit(ParentId=root_id, Name='ou02')
    client.create_organizational_unit(ParentId=root_id, Name='ou03')
    response = client.list_organizational_units_for_parent(ParentId=root_id)
    response.should.have.key('OrganizationalUnits').should.be.a(list)
    for ou in response['OrganizationalUnits']:
        validate_organizational_unit(org, dict(OrganizationalUnit=ou))


@mock_organizations
def test_list_organizational_units_for_parent_exception():
    client = boto3.client('organizations', region_name='us-east-1')
    with assert_raises(ClientError) as e:
        response = client.list_organizational_units_for_parent(
            ParentId=utils.make_random_root_id()
        )
    ex = e.exception
    ex.operation_name.should.equal('ListOrganizationalUnitsForParent')
    ex.response['Error']['Code'].should.equal('400')
    ex.response['Error']['Message'].should.contain('ParentNotFoundException')


# Accounts
mockname = 'mock-account'
mockdomain = 'moto-example.org'
mockemail = '@'.join([mockname, mockdomain])


@mock_organizations
def test_create_account():
    client = boto3.client('organizations', region_name='us-east-1')
    client.create_organization(FeatureSet='ALL')
    create_status = client.create_account(
        AccountName=mockname, Email=mockemail
    )['CreateAccountStatus']
    validate_create_account_status(create_status)
    create_status['AccountName'].should.equal(mockname)


@mock_organizations
def test_describe_account():
    client = boto3.client('organizations', region_name='us-east-1')
    org = client.create_organization(FeatureSet='ALL')['Organization']
    account_id = client.create_account(
        AccountName=mockname, Email=mockemail
    )['CreateAccountStatus']['AccountId']
    response = client.describe_account(AccountId=account_id)
    validate_account(org, response['Account'])
    response['Account']['Name'].should.equal(mockname)
    response['Account']['Email'].should.equal(mockemail)


@mock_organizations
def test_describe_account_exception():
    client = boto3.client('organizations', region_name='us-east-1')
    with assert_raises(ClientError) as e:
        response = client.describe_account(AccountId=utils.make_random_account_id())
    ex = e.exception
    ex.operation_name.should.equal('DescribeAccount')
    ex.response['Error']['Code'].should.equal('400')
    ex.response['Error']['Message'].should.contain('AccountNotFoundException')


@mock_organizations
def test_list_accounts():
    client = boto3.client('organizations', region_name='us-east-1')
    org = client.create_organization(FeatureSet='ALL')['Organization']
    for i in range(5):
        name = mockname + str(i)
        email = name + '@' + mockdomain
        client.create_account(AccountName=name, Email=email)
    response = client.list_accounts()
    response.should.have.key('Accounts')
    accounts = response['Accounts']
    len(accounts).should.equal(5)
    for account in accounts:
        validate_account(org, account)
    accounts[3]['Name'].should.equal(mockname + '3')
    accounts[2]['Email'].should.equal(mockname + '2' + '@' + mockdomain)


@mock_organizations
def test_list_accounts_for_parent():
    client = boto3.client('organizations', region_name='us-east-1')
    org = client.create_organization(FeatureSet='ALL')['Organization']
    root_id = client.list_roots()['Roots'][0]['Id']
    account_id = client.create_account(
        AccountName=mockname,
        Email=mockemail,
    )['CreateAccountStatus']['AccountId']
    response = client.list_accounts_for_parent(ParentId=root_id)
    account_id.should.be.within([account['Id'] for account in response['Accounts']])


@mock_organizations
def test_move_account():
    client = boto3.client('organizations', region_name='us-east-1')
    org = client.create_organization(FeatureSet='ALL')['Organization']
    root_id = client.list_roots()['Roots'][0]['Id']
    account_id = client.create_account(
        AccountName=mockname, Email=mockemail
    )['CreateAccountStatus']['AccountId']
    ou01 = client.create_organizational_unit(ParentId=root_id, Name='ou01')
    ou01_id = ou01['OrganizationalUnit']['Id']
    client.move_account(
        AccountId=account_id,
        SourceParentId=root_id,
        DestinationParentId=ou01_id,
    )
    response = client.list_accounts_for_parent(ParentId=ou01_id)
    account_id.should.be.within([account['Id'] for account in response['Accounts']])


@mock_organizations
def test_list_parents_for_ou():
    client = boto3.client('organizations', region_name='us-east-1')
    org = client.create_organization(FeatureSet='ALL')['Organization']
    root_id = client.list_roots()['Roots'][0]['Id']
    ou01 = client.create_organizational_unit(ParentId=root_id, Name='ou01')
    ou01_id = ou01['OrganizationalUnit']['Id']
    response01 = client.list_parents(ChildId=ou01_id)
    response01.should.have.key('Parents').should.be.a(list)
    response01['Parents'][0].should.have.key('Id').should.equal(root_id)
    response01['Parents'][0].should.have.key('Type').should.equal('ROOT')
    ou02 = client.create_organizational_unit(ParentId=ou01_id, Name='ou02')
    ou02_id = ou02['OrganizationalUnit']['Id']
    response02 = client.list_parents(ChildId=ou02_id)
    response02.should.have.key('Parents').should.be.a(list)
    response02['Parents'][0].should.have.key('Id').should.equal(ou01_id)
    response02['Parents'][0].should.have.key('Type').should.equal('ORGANIZATIONAL_UNIT')


@mock_organizations
def test_list_parents_for_accounts():
    client = boto3.client('organizations', region_name='us-east-1')
    org = client.create_organization(FeatureSet='ALL')['Organization']
    root_id = client.list_roots()['Roots'][0]['Id']
    ou01 = client.create_organizational_unit(ParentId=root_id, Name='ou01')
    ou01_id = ou01['OrganizationalUnit']['Id']
    account01_id = client.create_account(
        AccountName='account01',
        Email='account01@moto-example.org'
    )['CreateAccountStatus']['AccountId']
    account02_id = client.create_account(
        AccountName='account02',
        Email='account02@moto-example.org'
    )['CreateAccountStatus']['AccountId']
    client.move_account(
        AccountId=account02_id,
        SourceParentId=root_id,
        DestinationParentId=ou01_id,
    )
    response01 = client.list_parents(ChildId=account01_id)
    response01.should.have.key('Parents').should.be.a(list)
    response01['Parents'][0].should.have.key('Id').should.equal(root_id)
    response01['Parents'][0].should.have.key('Type').should.equal('ROOT')
    response02 = client.list_parents(ChildId=account02_id)
    response02.should.have.key('Parents').should.be.a(list)
    response02['Parents'][0].should.have.key('Id').should.equal(ou01_id)
    response02['Parents'][0].should.have.key('Type').should.equal('ORGANIZATIONAL_UNIT')


@mock_organizations
def test_list_children():
    client = boto3.client('organizations', region_name='us-east-1')
    org = client.create_organization(FeatureSet='ALL')['Organization']
    root_id = client.list_roots()['Roots'][0]['Id']
    ou01 = client.create_organizational_unit(ParentId=root_id, Name='ou01')
    ou01_id = ou01['OrganizationalUnit']['Id']
    ou02 = client.create_organizational_unit(ParentId=ou01_id, Name='ou02')
    ou02_id = ou02['OrganizationalUnit']['Id']
    account01_id = client.create_account(
        AccountName='account01',
        Email='account01@moto-example.org'
    )['CreateAccountStatus']['AccountId']
    account02_id = client.create_account(
        AccountName='account02',
        Email='account02@moto-example.org'
    )['CreateAccountStatus']['AccountId']
    client.move_account(
        AccountId=account02_id,
        SourceParentId=root_id,
        DestinationParentId=ou01_id,
    )
    response01 = client.list_children(ParentId=root_id, ChildType='ACCOUNT')
    response02 = client.list_children(ParentId=root_id, ChildType='ORGANIZATIONAL_UNIT')
    response03 = client.list_children(ParentId=ou01_id, ChildType='ACCOUNT')
    response04 = client.list_children(ParentId=ou01_id, ChildType='ORGANIZATIONAL_UNIT')
    response01['Children'][0]['Id'].should.equal(account01_id)
    response01['Children'][0]['Type'].should.equal('ACCOUNT')
    response02['Children'][0]['Id'].should.equal(ou01_id)
    response02['Children'][0]['Type'].should.equal('ORGANIZATIONAL_UNIT')
    response03['Children'][0]['Id'].should.equal(account02_id)
    response03['Children'][0]['Type'].should.equal('ACCOUNT')
    response04['Children'][0]['Id'].should.equal(ou02_id)
    response04['Children'][0]['Type'].should.equal('ORGANIZATIONAL_UNIT')


@mock_organizations
def test_list_children_exception():
    client = boto3.client('organizations', region_name='us-east-1')
    org = client.create_organization(FeatureSet='ALL')['Organization']
    root_id = client.list_roots()['Roots'][0]['Id']
    with assert_raises(ClientError) as e:
        response = client.list_children(
            ParentId=utils.make_random_root_id(),
            ChildType='ACCOUNT'
        )
    ex = e.exception
    ex.operation_name.should.equal('ListChildren')
    ex.response['Error']['Code'].should.equal('400')
    ex.response['Error']['Message'].should.contain('ParentNotFoundException')
    with assert_raises(ClientError) as e:
        response = client.list_children(
            ParentId=root_id,
            ChildType='BLEE'
        )
    ex = e.exception
    ex.operation_name.should.equal('ListChildren')
    ex.response['Error']['Code'].should.equal('400')
    ex.response['Error']['Message'].should.contain('InvalidInputException')


# Service Control Policies
policy_doc01 = dict(
    Version='2012-10-17',
    Statement=[dict(
        Sid='MockPolicyStatement',
        Effect='Allow',
        Action='s3:*',
        Resource='*',
    )]
)

@mock_organizations
def test_create_policy():
    client = boto3.client('organizations', region_name='us-east-1')
    org = client.create_organization(FeatureSet='ALL')['Organization']
    policy = client.create_policy(
        Content=json.dumps(policy_doc01),
        Description='A dummy service control policy',
        Name='MockServiceControlPolicy',
        Type='SERVICE_CONTROL_POLICY'
    )['Policy']
    validate_service_control_policy(org, policy)
    policy['PolicySummary']['Name'].should.equal('MockServiceControlPolicy')
    policy['PolicySummary']['Description'].should.equal('A dummy service control policy')
    policy['Content'].should.equal(json.dumps(policy_doc01))


@mock_organizations
def test_describe_policy():
    client = boto3.client('organizations', region_name='us-east-1')
    org = client.create_organization(FeatureSet='ALL')['Organization']
    policy_id = client.create_policy(
        Content=json.dumps(policy_doc01),
        Description='A dummy service control policy',
        Name='MockServiceControlPolicy',
        Type='SERVICE_CONTROL_POLICY'
    )['Policy']['PolicySummary']['Id']
    policy = client.describe_policy(PolicyId=policy_id)['Policy']
    validate_service_control_policy(org, policy)
    policy['PolicySummary']['Name'].should.equal('MockServiceControlPolicy')
    policy['PolicySummary']['Description'].should.equal('A dummy service control policy')
    policy['Content'].should.equal(json.dumps(policy_doc01))


@mock_organizations
def test_describe_policy_exception():
    client = boto3.client('organizations', region_name='us-east-1')
    client.create_organization(FeatureSet='ALL')['Organization']
    policy_id = 'p-47fhe9s3'
    with assert_raises(ClientError) as e:
        response = client.describe_policy(PolicyId=policy_id)
    ex = e.exception
    ex.operation_name.should.equal('DescribePolicy')
    ex.response['Error']['Code'].should.equal('400')
    ex.response['Error']['Message'].should.contain('PolicyNotFoundException')
    with assert_raises(ClientError) as e:
        response = client.describe_policy(PolicyId='meaninglessstring')
    ex = e.exception
    ex.operation_name.should.equal('DescribePolicy')
    ex.response['Error']['Code'].should.equal('400')
    ex.response['Error']['Message'].should.contain('InvalidInputException')


@mock_organizations
def test_attach_policy():
    client = boto3.client('organizations', region_name='us-east-1')
    org = client.create_organization(FeatureSet='ALL')['Organization']
    root_id = client.list_roots()['Roots'][0]['Id']
    ou_id = client.create_organizational_unit(
        ParentId=root_id,
        Name='ou01',
    )['OrganizationalUnit']['Id']
    account_id = client.create_account(
        AccountName=mockname,
        Email=mockemail,
    )['CreateAccountStatus']['AccountId']
    policy_id = client.create_policy(
        Content=json.dumps(policy_doc01),
        Description='A dummy service control policy',
        Name='MockServiceControlPolicy',
        Type='SERVICE_CONTROL_POLICY'
    )['Policy']['PolicySummary']['Id']
    response = client.attach_policy(PolicyId=policy_id, TargetId=root_id)
    response['ResponseMetadata']['HTTPStatusCode'].should.equal(200)
    response = client.attach_policy(PolicyId=policy_id, TargetId=ou_id)
    response['ResponseMetadata']['HTTPStatusCode'].should.equal(200)
    response = client.attach_policy(PolicyId=policy_id, TargetId=account_id)
    response['ResponseMetadata']['HTTPStatusCode'].should.equal(200)


@mock_organizations
def test_attach_policy_exception():
    client = boto3.client('organizations', region_name='us-east-1')
    client.create_organization(FeatureSet='ALL')['Organization']
    root_id='r-dj873'
    ou_id='ou-gi99-i7r8eh2i2'
    account_id='126644886543'
    policy_id = client.create_policy(
        Content=json.dumps(policy_doc01),
        Description='A dummy service control policy',
        Name='MockServiceControlPolicy',
        Type='SERVICE_CONTROL_POLICY'
    )['Policy']['PolicySummary']['Id']
    with assert_raises(ClientError) as e:
        response = client.attach_policy(PolicyId=policy_id, TargetId=root_id)
    ex = e.exception
    ex.operation_name.should.equal('AttachPolicy')
    ex.response['Error']['Code'].should.equal('400')
    ex.response['Error']['Message'].should.contain('OrganizationalUnitNotFoundException')
    with assert_raises(ClientError) as e:
        response = client.attach_policy(PolicyId=policy_id, TargetId=ou_id)
    ex = e.exception
    ex.operation_name.should.equal('AttachPolicy')
    ex.response['Error']['Code'].should.equal('400')
    ex.response['Error']['Message'].should.contain('OrganizationalUnitNotFoundException')
    with assert_raises(ClientError) as e:
        response = client.attach_policy(PolicyId=policy_id, TargetId=account_id)
    ex = e.exception
    ex.operation_name.should.equal('AttachPolicy')
    ex.response['Error']['Code'].should.equal('400')
    ex.response['Error']['Message'].should.contain('AccountNotFoundException')
    with assert_raises(ClientError) as e:
        response = client.attach_policy(PolicyId=policy_id, TargetId='meaninglessstring')
    ex = e.exception
    ex.operation_name.should.equal('AttachPolicy')
    ex.response['Error']['Code'].should.equal('400')
    ex.response['Error']['Message'].should.contain('InvalidInputException')


@mock_organizations
def test_list_polices():
    client = boto3.client('organizations', region_name='us-east-1')
    org = client.create_organization(FeatureSet='ALL')['Organization']
    for i in range(0,4):
        client.create_policy(
            Content=json.dumps(policy_doc01),
            Description='A dummy service control policy',
            Name='MockServiceControlPolicy' + str(i),
            Type='SERVICE_CONTROL_POLICY'
        )
    response = client.list_policies(Filter='SERVICE_CONTROL_POLICY')
    for policy in response['Policies']:
        validate_policy_summary(org, policy)
 

@mock_organizations
def test_list_policies_for_target():
    client = boto3.client('organizations', region_name='us-east-1')
    org = client.create_organization(FeatureSet='ALL')['Organization']
    root_id = client.list_roots()['Roots'][0]['Id']
    ou_id = client.create_organizational_unit(
        ParentId=root_id,
        Name='ou01',
    )['OrganizationalUnit']['Id']
    account_id = client.create_account(
        AccountName=mockname,
        Email=mockemail,
    )['CreateAccountStatus']['AccountId']
    policy_id = client.create_policy(
        Content=json.dumps(policy_doc01),
        Description='A dummy service control policy',
        Name='MockServiceControlPolicy',
        Type='SERVICE_CONTROL_POLICY'
    )['Policy']['PolicySummary']['Id']
    client.attach_policy(PolicyId=policy_id, TargetId=ou_id)
    response = client.list_policies_for_target(
        TargetId=ou_id,
        Filter='SERVICE_CONTROL_POLICY',
    )
    for policy in response['Policies']:
        validate_policy_summary(org, policy)
    client.attach_policy(PolicyId=policy_id, TargetId=account_id)
    response = client.list_policies_for_target(
        TargetId=account_id,
        Filter='SERVICE_CONTROL_POLICY',
    )
    for policy in response['Policies']:
        validate_policy_summary(org, policy)


@mock_organizations
def test_list_policies_for_target_exception():
    client = boto3.client('organizations', region_name='us-east-1')
    client.create_organization(FeatureSet='ALL')['Organization']
    ou_id='ou-gi99-i7r8eh2i2'
    account_id='126644886543'
    with assert_raises(ClientError) as e:
        response = client.list_policies_for_target(
            TargetId=ou_id,
            Filter='SERVICE_CONTROL_POLICY',
        )
    ex = e.exception
    ex.operation_name.should.equal('ListPoliciesForTarget')
    ex.response['Error']['Code'].should.equal('400')
    ex.response['Error']['Message'].should.contain('OrganizationalUnitNotFoundException')
    with assert_raises(ClientError) as e:
        response = client.list_policies_for_target(
            TargetId=account_id,
            Filter='SERVICE_CONTROL_POLICY',
        )
    ex = e.exception
    ex.operation_name.should.equal('ListPoliciesForTarget')
    ex.response['Error']['Code'].should.equal('400')
    ex.response['Error']['Message'].should.contain('AccountNotFoundException')
    with assert_raises(ClientError) as e:
        response = client.list_policies_for_target(
            TargetId='meaninglessstring',
            Filter='SERVICE_CONTROL_POLICY',
        )
    ex = e.exception
    ex.operation_name.should.equal('ListPoliciesForTarget')
    ex.response['Error']['Code'].should.equal('400')
    ex.response['Error']['Message'].should.contain('InvalidInputException')
 

@mock_organizations
def test_list_targets_for_policy():
    client = boto3.client('organizations', region_name='us-east-1')
    org = client.create_organization(FeatureSet='ALL')['Organization']
    root_id = client.list_roots()['Roots'][0]['Id']
    ou_id = client.create_organizational_unit(
        ParentId=root_id,
        Name='ou01',
    )['OrganizationalUnit']['Id']
    account_id = client.create_account(
        AccountName=mockname,
        Email=mockemail,
    )['CreateAccountStatus']['AccountId']
    policy_id = client.create_policy(
        Content=json.dumps(policy_doc01),
        Description='A dummy service control policy',
        Name='MockServiceControlPolicy',
        Type='SERVICE_CONTROL_POLICY'
    )['Policy']['PolicySummary']['Id']
    client.attach_policy(PolicyId=policy_id, TargetId=root_id)
    client.attach_policy(PolicyId=policy_id, TargetId=ou_id)
    client.attach_policy(PolicyId=policy_id, TargetId=account_id)
    response = client.list_targets_for_policy(PolicyId=policy_id)
    for target in response['Targets']:
        target.should.be.a(dict)
        target.should.have.key('Name').should.be.a(six.string_types)
        target.should.have.key('Arn').should.be.a(six.string_types)
        target.should.have.key('TargetId').should.be.a(six.string_types)
        target.should.have.key('Type').should.be.within(
            ['ROOT', 'ORGANIZATIONAL_UNIT', 'ACCOUNT']
        )


@mock_organizations
def test_list_targets_for_policy_exception():
    client = boto3.client('organizations', region_name='us-east-1')
    client.create_organization(FeatureSet='ALL')['Organization']
    policy_id = 'p-47fhe9s3'
    with assert_raises(ClientError) as e:
        response = client.list_targets_for_policy(PolicyId=policy_id)
    ex = e.exception
    ex.operation_name.should.equal('ListTargetsForPolicy')
    ex.response['Error']['Code'].should.equal('400')
    ex.response['Error']['Message'].should.contain('PolicyNotFoundException')
    with assert_raises(ClientError) as e:
        response = client.list_targets_for_policy(PolicyId='meaninglessstring')
    ex = e.exception
    ex.operation_name.should.equal('ListTargetsForPolicy')
    ex.response['Error']['Code'].should.equal('400')
    ex.response['Error']['Message'].should.contain('InvalidInputException')
 



from __future__ import unicode_literals

import sure  # noqa
import re
from nose.tools import assert_raises
import boto3
from botocore.client import ClientError


from datetime import datetime
import pytz

from moto import mock_glue
from . import helpers


@mock_glue
def test_create_database():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    helpers.create_database(client, database_name)

    response = helpers.get_database(client, database_name)
    database = response['Database']

    database.should.equal({'Name': database_name})


@mock_glue
def test_create_database_already_exists():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'cantcreatethisdatabasetwice'
    helpers.create_database(client, database_name)

    with assert_raises(ClientError) as exc:
        helpers.create_database(client, database_name)

    exc.exception.response['Error']['Code'].should.equal('AlreadyExistsException')


@mock_glue
def test_get_database_not_exits():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'nosuchdatabase'

    with assert_raises(ClientError) as exc:
        helpers.get_database(client, database_name)

    exc.exception.response['Error']['Code'].should.equal('EntityNotFoundException')
    exc.exception.response['Error']['Message'].should.match('Database nosuchdatabase not found')


@mock_glue
def test_create_table():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    helpers.create_database(client, database_name)

    table_name = 'myspecialtable'
    table_input = helpers.create_table_input(database_name, table_name)
    helpers.create_table(client, database_name, table_name, table_input)

    response = helpers.get_table(client, database_name, table_name)
    table = response['Table']

    table['Name'].should.equal(table_input['Name'])
    table['StorageDescriptor'].should.equal(table_input['StorageDescriptor'])
    table['PartitionKeys'].should.equal(table_input['PartitionKeys'])


@mock_glue
def test_create_table_already_exists():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    helpers.create_database(client, database_name)

    table_name = 'cantcreatethistabletwice'
    helpers.create_table(client, database_name, table_name)

    with assert_raises(ClientError) as exc:
        helpers.create_table(client, database_name, table_name)

    exc.exception.response['Error']['Code'].should.equal('AlreadyExistsException')


@mock_glue
def test_get_tables():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    helpers.create_database(client, database_name)

    table_names = ['myfirsttable', 'mysecondtable', 'mythirdtable']
    table_inputs = {}

    for table_name in table_names:
        table_input = helpers.create_table_input(database_name, table_name)
        table_inputs[table_name] = table_input
        helpers.create_table(client, database_name, table_name, table_input)

    response = helpers.get_tables(client, database_name)

    tables = response['TableList']

    tables.should.have.length_of(3)

    for table in tables:
        table_name = table['Name']
        table_name.should.equal(table_inputs[table_name]['Name'])
        table['StorageDescriptor'].should.equal(table_inputs[table_name]['StorageDescriptor'])
        table['PartitionKeys'].should.equal(table_inputs[table_name]['PartitionKeys'])


@mock_glue
def test_get_table_versions():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    helpers.create_database(client, database_name)

    table_name = 'myfirsttable'
    version_inputs = {}

    table_input = helpers.create_table_input(database_name, table_name)
    helpers.create_table(client, database_name, table_name, table_input)
    version_inputs["1"] = table_input

    columns = [{'Name': 'country', 'Type': 'string'}]
    table_input = helpers.create_table_input(database_name, table_name, columns=columns)
    helpers.update_table(client, database_name, table_name, table_input)
    version_inputs["2"] = table_input

    # Updateing with an indentical input should still create a new version
    helpers.update_table(client, database_name, table_name, table_input)
    version_inputs["3"] = table_input

    response = helpers.get_table_versions(client, database_name, table_name)

    vers = response['TableVersions']

    vers.should.have.length_of(3)
    vers[0]['Table']['StorageDescriptor']['Columns'].should.equal([])
    vers[-1]['Table']['StorageDescriptor']['Columns'].should.equal(columns)

    for n, ver in enumerate(vers):
        n = str(n + 1)
        ver['VersionId'].should.equal(n)
        ver['Table']['Name'].should.equal(table_name)
        ver['Table']['StorageDescriptor'].should.equal(version_inputs[n]['StorageDescriptor'])
        ver['Table']['PartitionKeys'].should.equal(version_inputs[n]['PartitionKeys'])

    response = helpers.get_table_version(client, database_name, table_name, "3")
    ver = response['TableVersion']

    ver['VersionId'].should.equal("3")
    ver['Table']['Name'].should.equal(table_name)
    ver['Table']['StorageDescriptor']['Columns'].should.equal(columns)


@mock_glue
def test_get_table_version_not_found():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    table_name = 'myfirsttable'
    helpers.create_database(client, database_name)
    helpers.create_table(client, database_name, table_name)

    with assert_raises(ClientError) as exc:
        helpers.get_table_version(client, database_name, 'myfirsttable', "20")

    exc.exception.response['Error']['Code'].should.equal('EntityNotFoundException')
    exc.exception.response['Error']['Message'].should.match('version', re.I)


@mock_glue
def test_get_table_version_invalid_input():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    table_name = 'myfirsttable'
    helpers.create_database(client, database_name)
    helpers.create_table(client, database_name, table_name)

    with assert_raises(ClientError) as exc:
        helpers.get_table_version(client, database_name, 'myfirsttable', "10not-an-int")

    exc.exception.response['Error']['Code'].should.equal('InvalidInputException')


@mock_glue
def test_get_table_not_exits():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    helpers.create_database(client, database_name)

    with assert_raises(ClientError) as exc:
        helpers.get_table(client, database_name, 'myfirsttable')

    exc.exception.response['Error']['Code'].should.equal('EntityNotFoundException')
    exc.exception.response['Error']['Message'].should.match('Table myfirsttable not found')


@mock_glue
def test_get_table_when_database_not_exits():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'nosuchdatabase'

    with assert_raises(ClientError) as exc:
        helpers.get_table(client, database_name, 'myfirsttable')

    exc.exception.response['Error']['Code'].should.equal('EntityNotFoundException')
    exc.exception.response['Error']['Message'].should.match('Database nosuchdatabase not found')


@mock_glue
def test_delete_table():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    helpers.create_database(client, database_name)

    table_name = 'myspecialtable'
    table_input = helpers.create_table_input(database_name, table_name)
    helpers.create_table(client, database_name, table_name, table_input)

    result = client.delete_table(DatabaseName=database_name, Name=table_name)
    result['ResponseMetadata']['HTTPStatusCode'].should.equal(200)

    # confirm table is deleted
    with assert_raises(ClientError) as exc:
        helpers.get_table(client, database_name, table_name)

    exc.exception.response['Error']['Code'].should.equal('EntityNotFoundException')
    exc.exception.response['Error']['Message'].should.match('Table myspecialtable not found')

@mock_glue
def test_batch_delete_table():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    helpers.create_database(client, database_name)

    table_name = 'myspecialtable'
    table_input = helpers.create_table_input(database_name, table_name)
    helpers.create_table(client, database_name, table_name, table_input)

    result = client.batch_delete_table(DatabaseName=database_name, TablesToDelete=[table_name])
    result['ResponseMetadata']['HTTPStatusCode'].should.equal(200)

    # confirm table is deleted
    with assert_raises(ClientError) as exc:
        helpers.get_table(client, database_name, table_name)

    exc.exception.response['Error']['Code'].should.equal('EntityNotFoundException')
    exc.exception.response['Error']['Message'].should.match('Table myspecialtable not found')


@mock_glue
def test_get_partitions_empty():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    table_name = 'myfirsttable'
    helpers.create_database(client, database_name)

    helpers.create_table(client, database_name, table_name)

    response = client.get_partitions(DatabaseName=database_name, TableName=table_name)

    response['Partitions'].should.have.length_of(0)


@mock_glue
def test_create_partition():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    table_name = 'myfirsttable'
    values = ['2018-10-01']
    helpers.create_database(client, database_name)

    helpers.create_table(client, database_name, table_name)

    before = datetime.now(pytz.utc)

    part_input = helpers.create_partition_input(database_name, table_name, values=values)
    helpers.create_partition(client, database_name, table_name, part_input)

    after = datetime.now(pytz.utc)

    response = client.get_partitions(DatabaseName=database_name, TableName=table_name)

    partitions = response['Partitions']

    partitions.should.have.length_of(1)

    partition = partitions[0]

    partition['TableName'].should.equal(table_name)
    partition['StorageDescriptor'].should.equal(part_input['StorageDescriptor'])
    partition['Values'].should.equal(values)
    partition['CreationTime'].should.be.greater_than(before)
    partition['CreationTime'].should.be.lower_than(after)


@mock_glue
def test_create_partition_already_exist():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    table_name = 'myfirsttable'
    values = ['2018-10-01']
    helpers.create_database(client, database_name)

    helpers.create_table(client, database_name, table_name)

    helpers.create_partition(client, database_name, table_name, values=values)

    with assert_raises(ClientError) as exc:
        helpers.create_partition(client, database_name, table_name, values=values)

    exc.exception.response['Error']['Code'].should.equal('AlreadyExistsException')


@mock_glue
def test_get_partition_not_found():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    table_name = 'myfirsttable'
    values = ['2018-10-01']
    helpers.create_database(client, database_name)

    helpers.create_table(client, database_name, table_name)

    with assert_raises(ClientError) as exc:
        helpers.get_partition(client, database_name, table_name, values)

    exc.exception.response['Error']['Code'].should.equal('EntityNotFoundException')
    exc.exception.response['Error']['Message'].should.match('partition')

@mock_glue
def test_batch_create_partition():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    table_name = 'myfirsttable'
    helpers.create_database(client, database_name)

    helpers.create_table(client, database_name, table_name)

    before = datetime.now(pytz.utc)

    partition_inputs = []
    for i in range(0, 20):
        values = ["2018-10-{:2}".format(i)]
        part_input = helpers.create_partition_input(database_name, table_name, values=values)
        partition_inputs.append(part_input)

    client.batch_create_partition(
        DatabaseName=database_name,
        TableName=table_name,
        PartitionInputList=partition_inputs
    )

    after = datetime.now(pytz.utc)

    response = client.get_partitions(DatabaseName=database_name, TableName=table_name)

    partitions = response['Partitions']

    partitions.should.have.length_of(20)

    for idx, partition in enumerate(partitions):
        partition_input = partition_inputs[idx]

        partition['TableName'].should.equal(table_name)
        partition['StorageDescriptor'].should.equal(partition_input['StorageDescriptor'])
        partition['Values'].should.equal(partition_input['Values'])
        partition['CreationTime'].should.be.greater_than(before)
        partition['CreationTime'].should.be.lower_than(after)


@mock_glue
def test_batch_create_partition_already_exist():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    table_name = 'myfirsttable'
    values = ['2018-10-01']
    helpers.create_database(client, database_name)

    helpers.create_table(client, database_name, table_name)

    helpers.create_partition(client, database_name, table_name, values=values)

    partition_input = helpers.create_partition_input(database_name, table_name, values=values)

    response = client.batch_create_partition(
        DatabaseName=database_name,
        TableName=table_name,
        PartitionInputList=[partition_input]
    )

    response.should.have.key('Errors')
    response['Errors'].should.have.length_of(1)
    response['Errors'][0]['PartitionValues'].should.equal(values)
    response['Errors'][0]['ErrorDetail']['ErrorCode'].should.equal('AlreadyExistsException')


@mock_glue
def test_get_partition():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    table_name = 'myfirsttable'
    helpers.create_database(client, database_name)

    helpers.create_table(client, database_name, table_name)

    values = [['2018-10-01'], ['2018-09-01']]

    helpers.create_partition(client, database_name, table_name, values=values[0])
    helpers.create_partition(client, database_name, table_name, values=values[1])

    response = client.get_partition(DatabaseName=database_name, TableName=table_name, PartitionValues=values[1])

    partition = response['Partition']

    partition['TableName'].should.equal(table_name)
    partition['Values'].should.equal(values[1])


@mock_glue
def test_update_partition_not_found_moving():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    table_name = 'myfirsttable'

    helpers.create_database(client, database_name)
    helpers.create_table(client, database_name, table_name)

    with assert_raises(ClientError) as exc:
        helpers.update_partition(client, database_name, table_name, old_values=['0000-00-00'], values=['2018-10-02'])

    exc.exception.response['Error']['Code'].should.equal('EntityNotFoundException')
    exc.exception.response['Error']['Message'].should.match('partition')


@mock_glue
def test_update_partition_not_found_change_in_place():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    table_name = 'myfirsttable'
    values = ['2018-10-01']

    helpers.create_database(client, database_name)
    helpers.create_table(client, database_name, table_name)

    with assert_raises(ClientError) as exc:
        helpers.update_partition(client, database_name, table_name, old_values=values, values=values)

    exc.exception.response['Error']['Code'].should.equal('EntityNotFoundException')
    exc.exception.response['Error']['Message'].should.match('partition')


@mock_glue
def test_update_partition_cannot_overwrite():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    table_name = 'myfirsttable'
    helpers.create_database(client, database_name)

    helpers.create_table(client, database_name, table_name)

    values = [['2018-10-01'], ['2018-09-01']]

    helpers.create_partition(client, database_name, table_name, values=values[0])
    helpers.create_partition(client, database_name, table_name, values=values[1])

    with assert_raises(ClientError) as exc:
        helpers.update_partition(client, database_name, table_name, old_values=values[0], values=values[1])

    exc.exception.response['Error']['Code'].should.equal('AlreadyExistsException')


@mock_glue
def test_update_partition():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    table_name = 'myfirsttable'
    values = ['2018-10-01']

    helpers.create_database(client, database_name)
    helpers.create_table(client, database_name, table_name)
    helpers.create_partition(client, database_name, table_name, values=values)

    response = helpers.update_partition(
        client,
        database_name,
        table_name,
        old_values=values,
        values=values,
        columns=[{'Name': 'country', 'Type': 'string'}],
    )

    response = client.get_partition(DatabaseName=database_name, TableName=table_name, PartitionValues=values)
    partition = response['Partition']

    partition['TableName'].should.equal(table_name)
    partition['StorageDescriptor']['Columns'].should.equal([{'Name': 'country', 'Type': 'string'}])


@mock_glue
def test_update_partition_move():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    table_name = 'myfirsttable'
    values = ['2018-10-01']
    new_values = ['2018-09-01']

    helpers.create_database(client, database_name)
    helpers.create_table(client, database_name, table_name)
    helpers.create_partition(client, database_name, table_name, values=values)

    response = helpers.update_partition(
        client,
        database_name,
        table_name,
        old_values=values,
        values=new_values,
        columns=[{'Name': 'country', 'Type': 'string'}],
    )

    with assert_raises(ClientError) as exc:
        helpers.get_partition(client, database_name, table_name, values)

    # Old partition shouldn't exist anymore
    exc.exception.response['Error']['Code'].should.equal('EntityNotFoundException')

    response = client.get_partition(DatabaseName=database_name, TableName=table_name, PartitionValues=new_values)
    partition = response['Partition']

    partition['TableName'].should.equal(table_name)
    partition['StorageDescriptor']['Columns'].should.equal([{'Name': 'country', 'Type': 'string'}])

@mock_glue
def test_delete_partition():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    table_name = 'myfirsttable'
    values = ['2018-10-01']
    helpers.create_database(client, database_name)
    helpers.create_table(client, database_name, table_name)

    part_input = helpers.create_partition_input(database_name, table_name, values=values)
    helpers.create_partition(client, database_name, table_name, part_input)

    client.delete_partition(
        DatabaseName=database_name,
        TableName=table_name,
        PartitionValues=values,
    )

    response = client.get_partitions(DatabaseName=database_name, TableName=table_name)
    partitions = response['Partitions']
    partitions.should.be.empty

@mock_glue
def test_delete_partition_bad_partition():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    table_name = 'myfirsttable'
    values = ['2018-10-01']
    helpers.create_database(client, database_name)
    helpers.create_table(client, database_name, table_name)

    with assert_raises(ClientError) as exc:
        client.delete_partition(
            DatabaseName=database_name,
            TableName=table_name,
            PartitionValues=values,
        )

    exc.exception.response['Error']['Code'].should.equal('EntityNotFoundException')

@mock_glue
def test_batch_delete_partition():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    table_name = 'myfirsttable'
    helpers.create_database(client, database_name)
    helpers.create_table(client, database_name, table_name)

    partition_inputs = []
    for i in range(0, 20):
        values = ["2018-10-{:2}".format(i)]
        part_input = helpers.create_partition_input(database_name, table_name, values=values)
        partition_inputs.append(part_input)

    client.batch_create_partition(
        DatabaseName=database_name,
        TableName=table_name,
        PartitionInputList=partition_inputs
    )

    partition_values = [{"Values": p["Values"]} for p in partition_inputs]

    response = client.batch_delete_partition(
        DatabaseName=database_name,
        TableName=table_name,
        PartitionsToDelete=partition_values,
    )

    response.should_not.have.key('Errors')

@mock_glue
def test_batch_delete_partition_with_bad_partitions():
    client = boto3.client('glue', region_name='us-east-1')
    database_name = 'myspecialdatabase'
    table_name = 'myfirsttable'
    helpers.create_database(client, database_name)
    helpers.create_table(client, database_name, table_name)

    partition_inputs = []
    for i in range(0, 20):
        values = ["2018-10-{:2}".format(i)]
        part_input = helpers.create_partition_input(database_name, table_name, values=values)
        partition_inputs.append(part_input)

    client.batch_create_partition(
        DatabaseName=database_name,
        TableName=table_name,
        PartitionInputList=partition_inputs
    )

    partition_values = [{"Values": p["Values"]} for p in partition_inputs]

    partition_values.insert(5, {"Values": ["2018-11-01"]})
    partition_values.insert(10, {"Values": ["2018-11-02"]})
    partition_values.insert(15, {"Values": ["2018-11-03"]})

    response = client.batch_delete_partition(
        DatabaseName=database_name,
        TableName=table_name,
        PartitionsToDelete=partition_values,
    )

    response.should.have.key('Errors')
    response['Errors'].should.have.length_of(3)
    error_partitions = map(lambda x: x['PartitionValues'], response['Errors'])
    ['2018-11-01'].should.be.within(error_partitions)
    ['2018-11-02'].should.be.within(error_partitions)
    ['2018-11-03'].should.be.within(error_partitions)



# #!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import mock

from moto.packages.httpretty.core import HTTPrettyRequest, fake_gethostname, fake_gethostbyname


def test_parse_querystring():

    core = HTTPrettyRequest(headers='test test HTTP/1.1')

    qs = 'test test'
    response = core.parse_querystring(qs)

    assert response == {}

def test_parse_request_body():
    core = HTTPrettyRequest(headers='test test HTTP/1.1')

    qs = 'test'
    response = core.parse_request_body(qs)

    assert response == 'test'

def test_fake_gethostname():

    response =  fake_gethostname()

    assert response == 'localhost'

def test_fake_gethostbyname():

    host = 'test'
    response = fake_gethostbyname(host=host)

    assert response == '127.0.0.1'